@@ -6,7 +6,7 @@
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
+   | available through the world-wide-web at the following url:           |
    | http://www.zend.com/license/2_00.txt.                                |
    | If you did not receive a copy of the Zend license and are unable to  |
    | obtain it through the world-wide-web, please send a note to          |
@@ -338,9 +338,14 @@ void zend_do_assign(znode *result, znode *variable, znode *value TSRMLS_DC)
 
 static inline zend_bool zend_is_function_or_method_call(znode *variable)
 {
-	zend_uint type = variable->u.EA.type;
-
-	return  (type == ZEND_PARSED_FCALL);
+	zend_uint type;
+	
+	if (variable->op_type != IS_VAR) {
+		return 0;
+	} else {
+		type = variable->u.EA.type;
+		return (type == ZEND_PARSED_FCALL);
+	}
 }
 
 
@@ -1121,7 +1126,7 @@ void zend_do_return(znode *expr, int do_end_vparse TSRMLS_DC)
 		INIT_ZVAL(opline->op1.u.constant);
 	}
 
-	if (do_end_vparse) {
+	if (expr) {
 		if (zend_is_function_or_method_call(expr)) {
 			opline->extended_value = ZEND_RETURNS_FUNCTION;	
 		} else {
