@@ -6,7 +6,7 @@
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
+   | available through the world-wide-web at the following url:           |
    | http://www.zend.com/license/2_00.txt.                                |
    | If you did not receive a copy of the Zend license and are unable to  |
    | obtain it through the world-wide-web, please send a note to          |
@@ -53,6 +53,7 @@
 
 /* These globals don't have to be thread safe since they're never modified */
 
+#define TEMP_VAR_STACK_LIMIT 2000
 
 /* Prototypes */
 static zval get_overloaded_property(temp_variable *T TSRMLS_DC);
@@ -265,12 +266,6 @@ void zend_assign_to_variable_reference(znode *result, zval **variable_ptr_ptr, z
 		variable_ptr_ptr = &EG(uninitialized_zval_ptr);
 /*	} else if (variable_ptr==&EG(uninitialized_zval) || variable_ptr!=value_ptr) { */
 	} else if (variable_ptr != value_ptr) {
-		variable_ptr->refcount--;
-		if (variable_ptr->refcount==0) {
-			zendi_zval_dtor(*variable_ptr);
-			FREE_ZVAL(variable_ptr);
-		}
-
 		if (!PZVAL_IS_REF(value_ptr)) {
 			/* break it away */
 			value_ptr->refcount--;
@@ -286,6 +281,12 @@ void zend_assign_to_variable_reference(znode *result, zval **variable_ptr_ptr, z
 
 		*variable_ptr_ptr = value_ptr;
 		value_ptr->refcount++;
+		
+		variable_ptr->refcount--;
+		if (variable_ptr->refcount==0) {
+			zendi_zval_dtor(*variable_ptr);
+			FREE_ZVAL(variable_ptr);
+		}
 	} else if (!variable_ptr->is_ref) {
 		if (variable_ptr_ptr == value_ptr_ptr) {
 			SEPARATE_ZVAL(variable_ptr_ptr);
@@ -335,6 +336,9 @@ static inline void zend_assign_to_variable(znode *result, znode *op1, znode *op2
 						AI_USE_PTR(Ts[result->u.var].var);
 					}
 
+					if(type == IS_TMP_VAR) {
+						value->refcount = value->is_ref = 0;
+					}
 					set_overloaded_property(&Ts[op1->u.var], value TSRMLS_CC);
 
 					if (!return_value_used && type == IS_TMP_VAR) {
@@ -665,8 +669,8 @@ fetch_string_dim:
 				}
 			}
 			break;
-		case IS_DOUBLE:
 		case IS_RESOURCE:
+		case IS_DOUBLE:
 		case IS_BOOL:
 		case IS_LONG: {
 				long index;
@@ -732,11 +736,15 @@ static void fetch_overloaded_element(znode *result, znode *op1, znode *op2, temp
 		return;
 	}
 
-	overloaded_element.element = *get_zval_ptr(op2, Ts, &EG(free_op2), type);
+	if(op2->op_type == IS_UNUSED && overloaded_element_type == OE_IS_ARRAY) {
+		ZVAL_NULL(&overloaded_element.element);
+	} else {
+		overloaded_element.element = *get_zval_ptr(op2, Ts, &EG(free_op2), type);
+		if (!EG(free_op2)) {
+			zval_copy_ctor(&overloaded_element.element);
+		}
+	} 
 	overloaded_element.type = overloaded_element_type;
-	if (!EG(free_op2)) {
-		zval_copy_ctor(&overloaded_element.element);
-	}
 
 	Ts[result->u.var].EA = Ts[op1->u.var].EA;
 	zend_llist_add_element(Ts[result->u.var].EA.data.overloaded_element.elements_list, &overloaded_element);
@@ -753,7 +761,6 @@ static void zend_fetch_dimension_address(znode *result, znode *op1, znode *op2,
 	zval *container;
 	zval ***retval = &Ts[result->u.var].var.ptr_ptr;
 
-
 	if (container_ptr == NULL) {
 		fetch_overloaded_element(result, op1, op2, Ts, type, retval, OE_IS_ARRAY TSRMLS_CC);
 		return;
@@ -794,7 +801,7 @@ static void zend_fetch_dimension_address(znode *result, znode *op1, znode *op2,
 				new_zval->refcount++;
 				if (zend_hash_next_index_insert(container->value.ht, &new_zval, sizeof(zval *), (void **) retval) == FAILURE) {
 					zend_error(E_WARNING, "Cannot add element to the array as the next element is already occupied");
-					*retval = &EG(uninitialized_zval_ptr);
+					*retval = &EG(error_zval_ptr);
 					new_zval->refcount--;
 				}
 			} else {
@@ -995,6 +1002,7 @@ static void call_overloaded_function(temp_variable *T, int arg_count, zval *retu
 	}
 	zend_llist_destroy(T->EA.data.overloaded_element.elements_list);
 	efree(T->EA.data.overloaded_element.elements_list);
+	PZVAL_UNLOCK(T->EA.data.overloaded_element.object);
 }
 
 
@@ -1038,7 +1046,11 @@ ZEND_API void execute(zend_op_array *op_array TSRMLS_DC)
 	EX(ce) = NULL;
 	EX(object).ptr = NULL;
 	EX(op_array) = op_array;
-	EX(Ts) = (temp_variable *) safe_emalloc(sizeof(temp_variable), op_array->T, 0);
+ 	if (op_array->T < TEMP_VAR_STACK_LIMIT) {
+ 		EX(Ts) = (temp_variable *) do_alloca(sizeof(temp_variable) * op_array->T);
+ 	} else {
+ 		EX(Ts) = (temp_variable *) safe_emalloc(sizeof(temp_variable), op_array->T, 0);
+ 	}
 	EX(prev_execute_data) = EG(current_execute_data);
 	EX(original_in_execution)=EG(in_execution);
 
@@ -1312,6 +1324,9 @@ binary_assign_op_addr: {
 					/* Behave like FETCH_DIM_W */
 					zend_fetch_dimension_address(&EX(opline)->result, &EX(opline)->op1, &EX(opline)->op2, EX(Ts), BP_VAR_W TSRMLS_CC);
 				} else {
+					if (EX(opline)->op2.op_type == IS_UNUSED) {
+						zend_error(E_ERROR, "Cannot use [] for reading");
+					}
 					/* Behave like FETCH_DIM_R, except for locking used for list() */
 					zend_fetch_dimension_address(&EX(opline)->result, &EX(opline)->op1, &EX(opline)->op2, EX(Ts), BP_VAR_R TSRMLS_CC);
 					AI_USE_PTR(EX(Ts)[EX(opline)->result.u.var].var);
@@ -1379,9 +1394,25 @@ binary_assign_op_addr: {
 					/* zend_assign_to_variable() always takes care of op2, never free it! */
 				}
 				NEXT_OPCODE();
-			case ZEND_ASSIGN_REF:
-				zend_assign_to_variable_reference(&EX(opline)->result, get_zval_ptr_ptr(&EX(opline)->op1, EX(Ts), BP_VAR_W), get_zval_ptr_ptr(&EX(opline)->op2, EX(Ts), BP_VAR_W), EX(Ts) TSRMLS_CC);
+			case ZEND_ASSIGN_REF: {
+				zval **value_ptr_ptr = get_zval_ptr_ptr(&EX(opline)->op2, EX(Ts), BP_VAR_W);
+
+				if (EX(opline)->op2.op_type == IS_VAR &&
+				    value_ptr_ptr &&
+				    !(*value_ptr_ptr)->is_ref &&
+				    EX(opline)->extended_value == ZEND_RETURNS_FUNCTION && 
+				    !EX(Ts)[EX(opline)->op2.u.var].var.fcall_returned_reference) {
+					zval *value;
+
+					PZVAL_LOCK(*value_ptr_ptr); /* undo the effect of get_zval_ptr_ptr() */
+					zend_error(E_NOTICE, "Only variables should be assigned by reference");
+					value = get_zval_ptr(&EX(opline)->op2, EX(Ts), &EG(free_op2), BP_VAR_R);
+ 					zend_assign_to_variable(&EX(opline)->result, &EX(opline)->op1, &EX(opline)->op2, value, (EG(free_op2)?IS_TMP_VAR:EX(opline)->op2.op_type), EX(Ts) TSRMLS_CC);
+				} else {
+					zend_assign_to_variable_reference(&EX(opline)->result, get_zval_ptr_ptr(&EX(opline)->op1, EX(Ts), BP_VAR_W), value_ptr_ptr, EX(Ts) TSRMLS_CC);
+				}
 				NEXT_OPCODE();
+			}
 			case ZEND_JMP:
 #if DEBUG_ZEND>=2
 				printf("Jumping to %d\n", EX(opline)->op1.u.opline_num);
@@ -1571,6 +1602,13 @@ binary_assign_op_addr: {
 									EX(Ts)[EX(opline)->op1.u.var].EA.data.overloaded_element.elements_list = (zend_llist *) emalloc(sizeof(zend_llist));
 									zend_llist_init(EX(Ts)[EX(opline)->op1.u.var].EA.data.overloaded_element.elements_list, sizeof(zend_overloaded_element), NULL, 0);
 								}
+								if(EX(Ts)[EX(opline)->op1.u.var].EA.data.overloaded_element.object) {
+									EX(Ts)[EX(opline)->op1.u.var].EA.data.overloaded_element.object->refcount++;
+									/* is-ref needed so that assign to this in call won't separate it */
+									if(EX(Ts)[EX(opline)->op1.u.var].EA.data.overloaded_element.object->refcount > 1) {
+										EX(Ts)[EX(opline)->op1.u.var].EA.data.overloaded_element.object->is_ref = 1;
+									}
+								}
 								zend_llist_add_element(EX(Ts)[EX(opline)->op1.u.var].EA.data.overloaded_element.elements_list, &overloaded_element);
 								EX(fbc) = (zend_function *) emalloc(sizeof(zend_function));
 								EX(fbc)->type = ZEND_OVERLOADED_FUNCTION;
@@ -1626,6 +1664,8 @@ do_fcall_common:
 
 					EX(Ts)[EX(opline)->result.u.var].var.ptr_ptr = &EX(Ts)[EX(opline)->result.u.var].var.ptr;
 
+					EX(Ts)[EX(opline)->result.u.var].var.fcall_returned_reference = 0;
+
 					if (EX(function_state).function->type==ZEND_INTERNAL_FUNCTION) {
 						ALLOC_ZVAL(EX(Ts)[EX(opline)->result.u.var].var.ptr);
 						INIT_ZVAL(*(EX(Ts)[EX(opline)->result.u.var].var.ptr));
@@ -1677,6 +1717,7 @@ do_fcall_common:
 						EG(active_op_array) = (zend_op_array *) EX(function_state).function;
 
 						zend_execute(EG(active_op_array) TSRMLS_CC);
+						EX(Ts)[EX(opline)->result.u.var].var.fcall_returned_reference = EG(active_op_array)->return_reference;
 
 						if (return_value_used && !EX(Ts)[EX(opline)->result.u.var].var.ptr) {
 							ALLOC_ZVAL(EX(Ts)[EX(opline)->result.u.var].var.ptr);
@@ -1718,15 +1759,30 @@ do_fcall_common:
 					zval *retval_ptr;
 					zval **retval_ptr_ptr;
 
-					if ((EG(active_op_array)->return_reference == ZEND_RETURN_REF) &&
-						(EX(opline)->op1.op_type != IS_CONST) &&
-						(EX(opline)->op1.op_type != IS_TMP_VAR)) {
+					if (EG(active_op_array)->return_reference == ZEND_RETURN_REF) {
+						if (EX(opline)->op1.op_type == IS_CONST || EX(opline)->op1.op_type == IS_TMP_VAR) {
+							/* Not supposed to happen, but we'll allow it */
+							zend_error(E_NOTICE, "Only variable references should be returned by reference");
+							goto return_by_value;
+						}
 
 						retval_ptr_ptr = get_zval_ptr_ptr(&EX(opline)->op1, EX(Ts), BP_VAR_W);
 
 						if (!retval_ptr_ptr) {
 							zend_error(E_ERROR, "Cannot return overloaded elements or string offsets by reference");
 						}
+
+						if (!(*retval_ptr_ptr)->is_ref) {
+							if (EX(opline)->extended_value == ZEND_RETURNS_FUNCTION &&
+								EX(Ts)[EX(opline)->op1.u.var].var.fcall_returned_reference) {
+								/* intentionally left empty */
+							} else if (EX(Ts)[EX(opline)->op1.u.var].var.ptr_ptr == &EX(Ts)[EX(opline)->op1.u.var].var.ptr) {
+								PZVAL_LOCK(*retval_ptr_ptr); /* undo the effect of get_zval_ptr_ptr() */
+								zend_error(E_NOTICE, "Only variable references should be returned by reference");
+								goto return_by_value;
+							}
+						}
+
 						SEPARATE_ZVAL_TO_MAKE_IS_REF(retval_ptr_ptr);
 
 #if 0  /* Will be fixed in ZE2, too dangerous to touch in the context of ZE1 */
@@ -1737,10 +1793,12 @@ do_fcall_common:
 						(*retval_ptr_ptr)->refcount++;
 						(*EG(return_value_ptr_ptr)) = (*retval_ptr_ptr);
 					} else {
+return_by_value:
 						retval_ptr = get_zval_ptr(&EX(opline)->op1, EX(Ts), &EG(free_op1), BP_VAR_R);
 
 						if (!EG(free_op1)) { /* Not a temp var */
-							if (PZVAL_IS_REF(retval_ptr) && retval_ptr->refcount > 0) {
+							if (EG(active_op_array)->return_reference == ZEND_RETURN_REF ||
+							    (PZVAL_IS_REF(retval_ptr) && retval_ptr->refcount > 0)) {
 								ALLOC_ZVAL(*(EG(return_value_ptr_ptr)));
 								**EG(return_value_ptr_ptr) = *retval_ptr;
 								(*EG(return_value_ptr_ptr))->is_ref = 0;
@@ -1757,7 +1815,11 @@ do_fcall_common:
 							(*EG(return_value_ptr_ptr))->is_ref = 0;
 						}
 					}
-					efree(EX(Ts));
+					if (EX(op_array)->T < TEMP_VAR_STACK_LIMIT) {
+						free_alloca(EX(Ts));
+					} else {
+						efree(EX(Ts));
+					}
 					EG(in_execution) = EX(original_in_execution);
 					EG(current_execute_data) = EX(prev_execute_data);
 					return;
@@ -1799,9 +1861,18 @@ do_fcall_common:
 						varptr->is_ref = 1;
 						varptr->refcount++;
 						zend_ptr_stack_push(&EG(argument_stack), varptr);
-						NEXT_OPCODE();
+					} else {
+						zval *valptr;
+
+						zend_error(E_NOTICE, "Only variables can be passed by reference");
+						ALLOC_ZVAL(valptr);
+						*valptr = *varptr;
+						if (!EG(free_op1)) {
+							zval_copy_ctor(valptr);
+						}
+						INIT_PZVAL(valptr);
+						zend_ptr_stack_push(&EG(argument_stack), valptr);
 					}
-					zend_error(E_ERROR, "Only variables can be passed by reference");
 				}
 				NEXT_OPCODE();
 			case ZEND_SEND_VAR:
@@ -2071,6 +2142,7 @@ send_by_ref:
 								zend_hash_update(array_ptr->value.ht, "", sizeof(""), &expr_ptr, sizeof(zval *), NULL);
 								break;
 							default:
+								zend_error(E_WARNING, "Illegal offset type");
 								zval_ptr_dtor(&expr_ptr);
 								/* do nothing */
 								break;
@@ -2118,7 +2190,8 @@ send_by_ref:
 					zend_op_array *new_op_array=NULL;
 					zval **original_return_value = EG(return_value_ptr_ptr);
 					int return_value_used;
-					zval *inc_filename = get_zval_ptr(&EX(opline)->op1, EX(Ts), &EG(free_op1), BP_VAR_R);
+					int free_op1;
+					zval *inc_filename = get_zval_ptr(&EX(opline)->op1, EX(Ts), &free_op1, BP_VAR_R);
 					zval tmp_inc_filename;
 					zend_bool failure_retval=0;
 
@@ -2180,7 +2253,6 @@ send_by_ref:
 					if (inc_filename==&tmp_inc_filename) {
 						zval_dtor(&tmp_inc_filename);
 					}
-					FREE_OP(EX(Ts), &EX(opline)->op1, EG(free_op1));
 					EX(Ts)[EX(opline)->result.u.var].var.ptr_ptr = &EX(Ts)[EX(opline)->result.u.var].var.ptr;
 					if (new_op_array) {
 						zend_class_entry *saved_ce;
@@ -2228,6 +2300,7 @@ send_by_ref:
 							EX(Ts)[EX(opline)->result.u.var].var.ptr->type = IS_BOOL;
 						}
 					}
+					FREE_OP(EX(Ts), &EX(opline)->op1, free_op1);
 					EG(return_value_ptr_ptr) = original_return_value;
 				}
 				NEXT_OPCODE();
@@ -2277,7 +2350,7 @@ send_by_ref:
 										long index;
 
 										if (offset->type == IS_DOUBLE) {
-											index = (long) offset->value.lval;
+											index = (long) offset->value.dval;
 										} else {
 											index = offset->value.lval;
 										}
@@ -2335,19 +2408,23 @@ send_by_ref:
 						/* probably redundant */
 						zend_hash_internal_pointer_reset(fe_ht);
 					} else {
-						/* JMP to the end of foreach - TBD */
+						zend_error(E_WARNING, "Invalid argument supplied for foreach()");
+						/* JMP to the end of foreach */
+						EX(opline) = op_array->opcodes+EX(opline)[1].op2.u.opline_num;
+						continue;
 					}
 				}
 				NEXT_OPCODE();
 			case ZEND_FE_FETCH: {
-					zval *array = get_zval_ptr(&EX(opline)->op1, EX(Ts), &EG(free_op1), BP_VAR_R);
-					zval *result = &EX(Ts)[EX(opline)->result.u.var].tmp_var;
+					zend_op *opline = EX(opline);
+					zval *array = get_zval_ptr(&opline->op1, EX(Ts), &EG(free_op1), BP_VAR_R);
 					zval **value, *key;
 					char *str_key;
 					uint str_key_len;
 					ulong int_key;
 					HashTable *fe_ht;
-
+					zend_bool use_key = opline->extended_value & ZEND_FE_FETCH_WITH_KEY;
+					zval *result;
 					PZVAL_LOCK(array);
 
 					fe_ht = HASH_OF(array);
@@ -2360,27 +2437,33 @@ send_by_ref:
 						EX(opline) = op_array->opcodes+EX(opline)->op2.u.opline_num;
 						continue;
 					}
-					array_init(result);
 
+					result = &EX(Ts)[opline->result.u.var].tmp_var;
+					if (!use_key) {
+						*result = **value;
+						zval_copy_ctor(result);
+					} else {
+						array_init(result);
 
-					(*value)->refcount++;
-					zend_hash_index_update(result->value.ht, 0, value, sizeof(zval *), NULL);
+						(*value)->refcount++;
+						zend_hash_index_update(result->value.ht, 0, value, sizeof(zval *), NULL);
 
-					ALLOC_ZVAL(key);
-					INIT_PZVAL(key);
-					switch (zend_hash_get_current_key_ex(fe_ht, &str_key, &str_key_len, &int_key, 1, NULL)) {
-						case HASH_KEY_IS_STRING:
-							key->value.str.val = str_key;
-							key->value.str.len = str_key_len-1;
-							key->type = IS_STRING;
-							break;
-						case HASH_KEY_IS_LONG:
-							key->value.lval = int_key;
-							key->type = IS_LONG;
-							break;
-						EMPTY_SWITCH_DEFAULT_CASE()
+						ALLOC_ZVAL(key);
+						INIT_PZVAL(key);
+						switch (zend_hash_get_current_key_ex(fe_ht, &str_key, &str_key_len, &int_key, 1, NULL)) {
+							case HASH_KEY_IS_STRING:
+								key->value.str.val = str_key;
+								key->value.str.len = str_key_len-1;
+								key->type = IS_STRING;
+								break;
+							case HASH_KEY_IS_LONG:
+								key->value.lval = int_key;
+								key->type = IS_LONG;
+								break;
+							EMPTY_SWITCH_DEFAULT_CASE()
+						}
+						zend_hash_index_update(result->value.ht, 1, &key, sizeof(zval *), NULL);
 					}
-					zend_hash_index_update(result->value.ht, 1, &key, sizeof(zval *), NULL);
 					zend_hash_move_forward(fe_ht);
 				}
 				NEXT_OPCODE();
