@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2001 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2003 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 0.92 of the Zend license,     |
-   | that is bundled with this package in the file LICENSE, and is        | 
-   | available at through the world-wide-web at                           |
-   | http://www.zend.com/license/0_92.txt.                                |
+   | This source file is subject to version 2.00 of the Zend license,     |
+   | that is bundled with this package in the file LICENSE, and is        |
+   | available through the world-wide-web at the following url:           |
+   | http://www.zend.com/license/2_00.txt.                                |
    | If you did not receive a copy of the Zend license and are unable to  |
    | obtain it through the world-wide-web, please send a note to          |
    | license@zend.com so we can mail you a copy immediately.              |
@@ -29,8 +29,11 @@
 #define HANDLE_NUMERIC(key, length, func) {												\
 	register char *tmp=key;																\
 																						\
+	if (*tmp=='-') {																	\
+		tmp++;																			\
+	}																					\
 	if ((*tmp>='0' && *tmp<='9')) do { /* possibly a numeric index */					\
-		char *end=tmp+length-1;															\
+		char *end=key+length-1;															\
 		ulong idx;																		\
 																						\
 		if (*tmp++=='0' && length>2) { /* don't accept numbers with leading zeros */	\
@@ -43,12 +46,19 @@
 			tmp++;																		\
 		}																				\
 		if (tmp==end && *tmp=='\0') { /* a numeric index */								\
-			idx = strtol(key, NULL, 10);													\
-			if (idx!=LONG_MAX) {														\
-				return func;															\
+			if (*key=='-') {															\
+				idx = strtol(key, NULL, 10);											\
+				if (idx!=LONG_MIN) {													\
+					return func;														\
+				}																		\
+			} else {																	\
+				idx = strtol(key, NULL, 10);											\
+				if (idx!=LONG_MAX) {													\
+					return func;														\
+				}																		\
 			}																			\
 		}																				\
-	} while (0);																			\
+	} while (0);																		\
 }
 
 
@@ -86,13 +96,13 @@ static void _zend_is_inconsistent(HashTable *ht, char *file, int line)
 	}
     switch (ht->inconsistent) {
 		case HT_IS_DESTROYING:
-			zend_output_debug_string(1, "%s(%d) : ht=0x%08x is being destroyed", file, line, ht);
+			zend_output_debug_string(1, "%s(%d) : ht=%p is being destroyed", file, line, ht);
 			break;
 		case HT_DESTROYED:
-			zend_output_debug_string(1, "%s(%d) : ht=0x%08x is already destroyed", file, line, ht);
+			zend_output_debug_string(1, "%s(%d) : ht=%p is already destroyed", file, line, ht);
 			break;
 		case HT_CLEANING:
-			zend_output_debug_string(1, "%s(%d) : ht=0x%08x is being cleaned", file, line, ht);
+			zend_output_debug_string(1, "%s(%d) : ht=%p is being cleaned", file, line, ht);
 			break;
     }
 	zend_bailout();
@@ -112,8 +122,10 @@ static void _zend_is_inconsistent(HashTable *ht, char *file, int line)
 	}
 
 
-#define HASH_UNPROTECT_RECURSION(ht)																\
-	(ht)->nApplyCount--;
+#define HASH_UNPROTECT_RECURSION(ht)													\
+	if ((ht)->bApplyProtection) {														\
+		(ht)->nApplyCount--;															\
+	}
 
 
 #define ZEND_HASH_IF_FULL_DO_RESIZE(ht)				\
@@ -162,6 +174,7 @@ ZEND_API ulong zend_hash_func(char *arKey, uint nKeyLength)
 ZEND_API int zend_hash_init(HashTable *ht, uint nSize, hash_func_t pHashFunction, dtor_func_t pDestructor, int persistent)
 {
 	uint i = 3;
+	Bucket **tmp;
 
 	SET_INCONSISTENT(HT_OK);
 
@@ -171,14 +184,6 @@ ZEND_API int zend_hash_init(HashTable *ht, uint nSize, hash_func_t pHashFunction
 
 	ht->nTableSize = 1 << i;
 	ht->nTableMask = ht->nTableSize - 1;
-	
-	/* Uses ecalloc() so that Bucket* == NULL */
-	ht->arBuckets = (Bucket **) pecalloc(ht->nTableSize, sizeof(Bucket *), persistent);
-	
-	if (!ht->arBuckets) {
-		return FAILURE;
-	}
-
 	ht->pDestructor = pDestructor;
 	ht->pListHead = NULL;
 	ht->pListTail = NULL;
@@ -188,6 +193,16 @@ ZEND_API int zend_hash_init(HashTable *ht, uint nSize, hash_func_t pHashFunction
 	ht->persistent = persistent;
 	ht->nApplyCount = 0;
 	ht->bApplyProtection = 1;
+	ht->arBuckets = NULL;
+
+	/* Uses ecalloc() so that Bucket* == NULL */
+	tmp = (Bucket **) pecalloc(ht->nTableSize, sizeof(Bucket *), persistent);
+
+	if (!tmp) {
+		return FAILURE;
+	}
+	ht->arBuckets = tmp;
+
 	return SUCCESS;
 }
 
@@ -224,7 +239,7 @@ ZEND_API int zend_hash_add_or_update(HashTable *ht, char *arKey, uint nKeyLength
 	}
 
 	HANDLE_NUMERIC(arKey, nKeyLength, zend_hash_index_update_or_next_insert(ht, idx, pData, nDataSize, pDest, flag));
-	
+
 	h = zend_inline_hash_func(arKey, nKeyLength);
 	nIndex = h & ht->nTableMask;
 
@@ -256,7 +271,7 @@ ZEND_API int zend_hash_add_or_update(HashTable *ht, char *arKey, uint nKeyLength
 		}
 		p = p->pNext;
 	}
-	
+
 	p = (Bucket *) pemalloc(sizeof(Bucket)-1+nKeyLength, ht->persistent);
 	if (!p) {
 		return FAILURE;
@@ -295,7 +310,7 @@ ZEND_API int zend_hash_quick_add_or_update(HashTable *ht, char *arKey, uint nKey
 	}
 
 	nIndex = h & ht->nTableMask;
-	
+
 	p = ht->arBuckets[nIndex];
 	while (p != NULL) {
 		if ((p->h == h) && (p->nKeyLength == nKeyLength)) {
@@ -324,7 +339,7 @@ ZEND_API int zend_hash_quick_add_or_update(HashTable *ht, char *arKey, uint nKey
 		}
 		p = p->pNext;
 	}
-	
+
 	p = (Bucket *) pemalloc(sizeof(Bucket)-1+nKeyLength, ht->persistent);
 	if (!p) {
 		return FAILURE;
@@ -334,7 +349,7 @@ ZEND_API int zend_hash_quick_add_or_update(HashTable *ht, char *arKey, uint nKey
 	p->nKeyLength = nKeyLength;
 	INIT_DATA(ht, p, pData, nDataSize);
 	p->h = h;
-	
+
 	CONNECT_TO_BUCKET_DLLIST(p, ht->arBuckets[nIndex]);
 
 	if (pDest) {
@@ -391,7 +406,7 @@ ZEND_API int zend_hash_index_update_or_next_insert(HashTable *ht, ulong h, void
 			}
 			UPDATE_DATA(ht, p, pData, nDataSize);
 			HANDLE_UNBLOCK_INTERRUPTIONS();
-			if (h >= ht->nNextFreeElement) {
+			if ((long)h >= (long)ht->nNextFreeElement) { /* comparing signed to avoid wraparounds on -1 */
 				ht->nNextFreeElement = h + 1;
 			}
 			if (pDest) {
@@ -419,7 +434,7 @@ ZEND_API int zend_hash_index_update_or_next_insert(HashTable *ht, ulong h, void
 	CONNECT_TO_GLOBAL_DLLIST(p, ht);
 	HANDLE_UNBLOCK_INTERRUPTIONS();
 
-	if (h >= ht->nNextFreeElement) {
+	if ((long)h >= (long)ht->nNextFreeElement) { /* comparing signed to avoid wraparounds on -1 */
 		ht->nNextFreeElement = h + 1;
 	}
 	ht->nNumOfElements++;
@@ -496,7 +511,7 @@ ZEND_API int zend_hash_del_key_or_index(HashTable *ht, char *arKey, uint nKeyLen
 			}
 			if (p->pListLast != NULL) {
 				p->pListLast->pListNext = p->pListNext;
-			} else { 
+			} else {
 				/* Deleting the head of the list */
 				ht->pListHead = p->pListNext;
 			}
@@ -616,7 +631,7 @@ static Bucket *zend_hash_apply_deleter(HashTable *ht, Bucket *p)
 
 	if (p->pListLast != NULL) {
 		p->pListLast->pListNext = p->pListNext;
-	} else { 
+	} else {
 		/* Deleting the head of the list */
 		ht->pListHead = p->pListNext;
 	}
@@ -651,8 +666,25 @@ ZEND_API void zend_hash_graceful_destroy(HashTable *ht)
 	SET_INCONSISTENT(HT_DESTROYED);
 }
 
+ZEND_API void zend_hash_graceful_reverse_destroy(HashTable *ht)
+{
+	Bucket *p;
+
+	IS_CONSISTENT(ht);
+
+	p = ht->pListTail;
+	while (p != NULL) {
+		zend_hash_apply_deleter(ht, p);
+		p = ht->pListTail;
+	}
+
+	pefree(ht->arBuckets, ht->persistent);
+
+	SET_INCONSISTENT(HT_DESTROYED);
+}
+
 /* This is used to selectively delete certain entries from a hashtable.
- * destruct() receives the data and decides if the entry should be deleted 
+ * destruct() receives the data and decides if the entry should be deleted
  * or not
  */
 
@@ -705,9 +737,9 @@ ZEND_API void zend_hash_apply_with_arguments(HashTable *ht, apply_func_args_t de
 
 	HASH_PROTECT_RECURSION(ht);
 
-	va_start(args, num_args);
 	p = ht->pListHead;
 	while (p != NULL) {
+		va_start(args, num_args);
 		hash_key.arKey = p->arKey;
 		hash_key.nKeyLength = p->nKeyLength;
 		hash_key.h = p->h;
@@ -716,8 +748,8 @@ ZEND_API void zend_hash_apply_with_arguments(HashTable *ht, apply_func_args_t de
 		} else {
 			p = p->pListNext;
 		}
+		va_end(args);
 	}
-	va_end(args);
 
 	HASH_UNPROTECT_RECURSION(ht);
 }
@@ -981,7 +1013,7 @@ ZEND_API void zend_hash_internal_pointer_reset_ex(HashTable *ht, HashPosition *p
 }
 
 
-/* This function will be extremely optimized by remembering 
+/* This function will be extremely optimized by remembering
  * the end of the list
  */
 ZEND_API void zend_hash_internal_pointer_end_ex(HashTable *ht, HashPosition *pos)
@@ -1026,7 +1058,7 @@ ZEND_API int zend_hash_move_backwards_ex(HashTable *ht, HashPosition *pos)
 ZEND_API int zend_hash_get_current_key_ex(HashTable *ht, char **str_index, uint *str_length, ulong *num_index, zend_bool duplicate, HashPosition *pos)
 {
 	Bucket *p;
-   
+
 	p = pos ? (*pos) : ht->pInternalPointer;
 
 	IS_CONSISTENT(ht);
@@ -1034,7 +1066,7 @@ ZEND_API int zend_hash_get_current_key_ex(HashTable *ht, char **str_index, uint
 	if (p) {
 		if (p->nKeyLength) {
 			if (duplicate) {
-				*str_index = estrndup(p->arKey, p->nKeyLength);
+				*str_index = estrndup(p->arKey, p->nKeyLength-1);
 			} else {
 				*str_index = p->arKey;
 			}
@@ -1054,7 +1086,7 @@ ZEND_API int zend_hash_get_current_key_ex(HashTable *ht, char **str_index, uint
 ZEND_API int zend_hash_get_current_key_type_ex(HashTable *ht, HashPosition *pos)
 {
 	Bucket *p;
-   
+
 	p = pos ? (*pos) : ht->pInternalPointer;
 
 	IS_CONSISTENT(ht);
@@ -1073,7 +1105,7 @@ ZEND_API int zend_hash_get_current_key_type_ex(HashTable *ht, HashPosition *pos)
 ZEND_API int zend_hash_get_current_data_ex(HashTable *ht, void **pData, HashPosition *pos)
 {
 	Bucket *p;
-   
+
 	p = pos ? (*pos) : ht->pInternalPointer;
 
 	IS_CONSISTENT(ht);
@@ -1088,7 +1120,7 @@ ZEND_API int zend_hash_get_current_data_ex(HashTable *ht, void **pData, HashPosi
 
 
 ZEND_API int zend_hash_sort(HashTable *ht, sort_func_t sort_func,
-							compare_func_t compar, int renumber)
+							compare_func_t compar, int renumber TSRMLS_DC)
 {
 	Bucket **arTmp;
 	Bucket *p;
@@ -1096,7 +1128,7 @@ ZEND_API int zend_hash_sort(HashTable *ht, sort_func_t sort_func,
 
 	IS_CONSISTENT(ht);
 
-	if (ht->nNumOfElements <= 1) {	/* Doesn't require sorting */
+	if (!(ht->nNumOfElements>1) && !(renumber && ht->nNumOfElements>0)) { /* Doesn't require sorting */
 		return SUCCESS;
 	}
 	arTmp = (Bucket **) pemalloc(ht->nNumOfElements * sizeof(Bucket *), ht->persistent);
@@ -1111,7 +1143,7 @@ ZEND_API int zend_hash_sort(HashTable *ht, sort_func_t sort_func,
 		i++;
 	}
 
-	(*sort_func)((void *) arTmp, i, sizeof(Bucket *), compar);
+	(*sort_func)((void *) arTmp, i, sizeof(Bucket *), compar TSRMLS_CC);
 
 	HANDLE_BLOCK_INTERRUPTIONS();
 	ht->pListHead = arTmp[0];
@@ -1153,13 +1185,13 @@ ZEND_API int zend_hash_compare(HashTable *ht1, HashTable *ht2, compare_func_t co
 	IS_CONSISTENT(ht1);
 	IS_CONSISTENT(ht2);
 
-	HASH_PROTECT_RECURSION(ht1); 
-	HASH_PROTECT_RECURSION(ht2); 
+	HASH_PROTECT_RECURSION(ht1);
+	HASH_PROTECT_RECURSION(ht2);
 
 	result = ht1->nNumOfElements - ht2->nNumOfElements;
 	if (result!=0) {
-		HASH_UNPROTECT_RECURSION(ht1); 
-		HASH_UNPROTECT_RECURSION(ht2); 
+		HASH_UNPROTECT_RECURSION(ht1);
+		HASH_UNPROTECT_RECURSION(ht2);
 		return result;
 	}
 
@@ -1170,29 +1202,29 @@ ZEND_API int zend_hash_compare(HashTable *ht1, HashTable *ht2, compare_func_t co
 
 	while (p1) {
 		if (ordered && !p2) {
-			HASH_UNPROTECT_RECURSION(ht1); 
-			HASH_UNPROTECT_RECURSION(ht2); 
+			HASH_UNPROTECT_RECURSION(ht1);
+			HASH_UNPROTECT_RECURSION(ht2);
 			return 1; /* That's not supposed to happen */
 		}
 		if (ordered) {
 			if (p1->nKeyLength==0 && p2->nKeyLength==0) { /* numeric indices */
 				result = p1->h - p2->h;
 				if (result!=0) {
-					HASH_UNPROTECT_RECURSION(ht1); 
-					HASH_UNPROTECT_RECURSION(ht2); 
+					HASH_UNPROTECT_RECURSION(ht1);
+					HASH_UNPROTECT_RECURSION(ht2);
 					return result;
 				}
 			} else { /* string indices */
 				result = p1->nKeyLength - p2->nKeyLength;
 				if (result!=0) {
-					HASH_UNPROTECT_RECURSION(ht1); 
-					HASH_UNPROTECT_RECURSION(ht2); 
+					HASH_UNPROTECT_RECURSION(ht1);
+					HASH_UNPROTECT_RECURSION(ht2);
 					return result;
 				}
 				result = memcmp(p1->arKey, p2->arKey, p1->nKeyLength);
 				if (result!=0) {
-					HASH_UNPROTECT_RECURSION(ht1); 
-					HASH_UNPROTECT_RECURSION(ht2); 
+					HASH_UNPROTECT_RECURSION(ht1);
+					HASH_UNPROTECT_RECURSION(ht2);
 					return result;
 				}
 			}
@@ -1200,22 +1232,22 @@ ZEND_API int zend_hash_compare(HashTable *ht1, HashTable *ht2, compare_func_t co
 		} else {
 			if (p1->nKeyLength==0) { /* numeric index */
 				if (zend_hash_index_find(ht2, p1->h, &pData2)==FAILURE) {
-					HASH_UNPROTECT_RECURSION(ht1); 
-					HASH_UNPROTECT_RECURSION(ht2); 
+					HASH_UNPROTECT_RECURSION(ht1);
+					HASH_UNPROTECT_RECURSION(ht2);
 					return 1;
 				}
 			} else { /* string index */
 				if (zend_hash_find(ht2, p1->arKey, p1->nKeyLength, &pData2)==FAILURE) {
-					HASH_UNPROTECT_RECURSION(ht1); 
-					HASH_UNPROTECT_RECURSION(ht2); 
+					HASH_UNPROTECT_RECURSION(ht1);
+					HASH_UNPROTECT_RECURSION(ht2);
 					return 1;
 				}
 			}
 		}
-		result = compar(p1->pData, pData2);
+		result = compar(p1->pData, pData2 TSRMLS_CC);
 		if (result!=0) {
-			HASH_UNPROTECT_RECURSION(ht1); 
-			HASH_UNPROTECT_RECURSION(ht2); 
+			HASH_UNPROTECT_RECURSION(ht1);
+			HASH_UNPROTECT_RECURSION(ht2);
 			return result;
 		}
 		p1 = p1->pListNext;
@@ -1223,14 +1255,14 @@ ZEND_API int zend_hash_compare(HashTable *ht1, HashTable *ht2, compare_func_t co
 			p2 = p2->pListNext;
 		}
 	}
-	
-	HASH_UNPROTECT_RECURSION(ht1); 
-	HASH_UNPROTECT_RECURSION(ht2); 
+
+	HASH_UNPROTECT_RECURSION(ht1);
+	HASH_UNPROTECT_RECURSION(ht2);
 	return 0;
 }
 
 
-ZEND_API int zend_hash_minmax(HashTable *ht, int (*compar) (const void *, const void *), int flag, void **pData)
+ZEND_API int zend_hash_minmax(HashTable *ht, compare_func_t compar, int flag, void **pData TSRMLS_DC)
 {
 	Bucket *p, *res;
 
@@ -1244,11 +1276,11 @@ ZEND_API int zend_hash_minmax(HashTable *ht, int (*compar) (const void *, const
 	res = p = ht->pListHead;
 	while ((p = p->pListNext)) {
 		if (flag) {
-			if (compar(&res, &p) < 0) { /* max */
+			if (compar(&res, &p TSRMLS_CC) < 0) { /* max */
 				res = p;
 			}
 		} else {
-			if (compar(&res, &p) > 0) { /* min */
+			if (compar(&res, &p TSRMLS_CC) > 0) { /* min */
 				res = p;
 			}
 		}
@@ -1286,14 +1318,14 @@ void zend_hash_display(HashTable *ht)
 	for (i = 0; i < ht->nTableSize; i++) {
 		p = ht->arBuckets[i];
 		while (p != NULL) {
-			zend_output_debug_string(0, "%s <==> 0x%X\n", p->arKey, p->h);
+			zend_output_debug_string(0, "%s <==> 0x%lX\n", p->arKey, p->h);
 			p = p->pNext;
 		}
 	}
 
 	p = ht->pListTail;
 	while (p != NULL) {
-		zend_output_debug_string(0, "%s <==> 0x%X\n", p->arKey, p->h);
+		zend_output_debug_string(0, "%s <==> 0x%lX\n", p->arKey, p->h);
 		p = p->pListLast;
 	}
 }
