@@ -1,52 +1,71 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP version 4.0                                                      |
+   | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2001 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 2.02 of the PHP license,      |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
-   | http://www.php.net/license/2_02.txt.                                 |
+   | available through the world-wide-web at the following url:           |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Authors: Jim Winstead (jimw@php.net)                                 |
-   |          Stig Sther Bakken <ssb@guardian.no>                        |
+   | Authors: Jim Winstead <jimw@php.net>                                 |
+   |          Stig Sther Bakken <ssb@fast.no>                            |
    |          Zeev Suraski <zeev@zend.com>                                |
-   | PHP 4.0 patches by Thies C. Arntzen (thies@thieso.net)               |
+   | PHP 4.0 patches by Thies C. Arntzen <thies@thieso.net>               |
    +----------------------------------------------------------------------+
 */
 
-/* $Id: math.c,v 1.53.2.5 2001/12/16 11:05:28 derick Exp $ */
+/* $Id: math.c,v 1.97.2.13.2.3 2006/01/01 13:46:57 sniper Exp $ */
 
 #include "php.h"
 #include "php_math.h"
+#include "zend_multiply.h"
 
 #include <math.h>
 #include <float.h>
+#include <stdlib.h>
 
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
 #endif
 
+#ifndef PHP_ROUND_FUZZ
+# ifndef PHP_WIN32
+#  define PHP_ROUND_FUZZ 0.50000000001
+# else
+#  define PHP_ROUND_FUZZ 0.5
+# endif
+#endif
 
-char *_php_math_number_format(double, int, char , char);
+#define PHP_ROUND_WITH_FUZZ(val, places) {			\
+	double tmp_val=val, f = pow(10.0, (double) places);	\
+	tmp_val *= f;					\
+	if (tmp_val >= 0.0) {				\
+		tmp_val = floor(tmp_val + PHP_ROUND_FUZZ);	\
+	} else {					\
+		tmp_val = ceil(tmp_val - PHP_ROUND_FUZZ);	\
+	}						\
+	tmp_val /= f;					\
+	val = !zend_isnan(tmp_val) ? tmp_val : val;	\
+}							\
 
 /* {{{ proto int abs(int number)
-
    Return the absolute value of the number */
-PHP_FUNCTION(abs) 
+
+PHP_FUNCTION(abs)
 {
 	zval **value;
-	
+
 	if (ZEND_NUM_ARGS()!=1||zend_get_parameters_ex(1, &value)==FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
 
 	convert_scalar_to_number_ex(value);
-	
+
 	if (Z_TYPE_PP(value) == IS_DOUBLE) {
 		RETURN_DOUBLE(fabs(Z_DVAL_PP(value)));
 	} else if (Z_TYPE_PP(value) == IS_LONG) {
@@ -60,13 +79,13 @@ PHP_FUNCTION(abs)
 	RETURN_FALSE;
 }
 
-/* }}} */ 
-/* {{{ proto double ceil(double number)
+/* }}} */
+/* {{{ proto float ceil(float number)
    Returns the next highest integer value of the number */
-PHP_FUNCTION(ceil) 
+PHP_FUNCTION(ceil)
 {
 	zval **value;
-	
+
 	if (ZEND_NUM_ARGS()!=1||zend_get_parameters_ex(1, &value)==FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
@@ -76,20 +95,20 @@ PHP_FUNCTION(ceil)
 	if (Z_TYPE_PP(value) == IS_DOUBLE) {
 		RETURN_DOUBLE(ceil(Z_DVAL_PP(value)));
 	} else if (Z_TYPE_PP(value) == IS_LONG) {
-		RETURN_LONG(Z_LVAL_PP(value));
+		convert_to_double_ex(value);
+		RETURN_DOUBLE(Z_DVAL_PP(value));
 	}
 
 	RETURN_FALSE;
 }
 
 /* }}} */
-/* {{{ proto double floor(double number)
+/* {{{ proto float floor(float number)
    Returns the next lowest integer value from the number */
-
 PHP_FUNCTION(floor)
 {
 	zval **value;
-	
+
 	if (ZEND_NUM_ARGS()!=1||zend_get_parameters_ex(1, &value)==FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
@@ -99,7 +118,8 @@ PHP_FUNCTION(floor)
 	if (Z_TYPE_PP(value) == IS_DOUBLE) {
 		RETURN_DOUBLE(floor(Z_DVAL_PP(value)));
 	} else if (Z_TYPE_PP(value) == IS_LONG) {
-		RETURN_LONG(Z_LVAL_PP(value));
+		convert_to_double_ex(value);
+		RETURN_DOUBLE(Z_DVAL_PP(value));
 	}
 
 	RETURN_FALSE;
@@ -108,14 +128,14 @@ PHP_FUNCTION(floor)
 /* }}} */
 
 
-/* {{{ proto double round(double number [, int precision])
-   Returns the number rounded to specified precision. */
+/* {{{ proto float round(float number [, int precision])
+   Returns the number rounded to specified precision */
 PHP_FUNCTION(round)
 {
 	zval **value, **precision;
 	int places = 0;
-	double f, return_val;
-	
+	double return_val;
+
 	if (ZEND_NUM_ARGS() < 1 || ZEND_NUM_ARGS() > 2 ||
 		zend_get_parameters_ex(ZEND_NUM_ARGS(), &value, &precision) == FAILURE) {
 		WRONG_PARAM_COUNT;
@@ -140,14 +160,7 @@ PHP_FUNCTION(round)
 			return_val = (Z_TYPE_PP(value) == IS_LONG) ?
 							(double)Z_LVAL_PP(value) : Z_DVAL_PP(value);
 
-			f = pow(10.0, places);
-
-			return_val *= f;
-			if (return_val >= 0.0)
-				return_val = floor(return_val + 0.5);
-			else
-				return_val = ceil(return_val - 0.5);
-			return_val /= f;
+			PHP_ROUND_WITH_FUZZ(return_val, places);
 
 			RETURN_DOUBLE(return_val);
 			break;
@@ -158,7 +171,7 @@ PHP_FUNCTION(round)
 	}
 }
 /* }}} */
-/* {{{ proto double sin(double number)
+/* {{{ proto float sin(float number)
    Returns the sine of the number in radians */
 
 PHP_FUNCTION(sin)
@@ -174,7 +187,7 @@ PHP_FUNCTION(sin)
 }
 
 /* }}} */
-/* {{{ proto double cos(double number)
+/* {{{ proto float cos(float number)
    Returns the cosine of the number in radians */
 
 PHP_FUNCTION(cos)
@@ -189,7 +202,7 @@ PHP_FUNCTION(cos)
 	Z_TYPE_P(return_value) = IS_DOUBLE;
 }
 /* }}} */
-/* {{{ proto double tan(double number)
+/* {{{ proto float tan(float number)
    Returns the tangent of the number in radians */
 PHP_FUNCTION(tan)
 {
@@ -204,7 +217,7 @@ PHP_FUNCTION(tan)
 }
 
 /* }}} */
-/* {{{ proto double asin(double number)
+/* {{{ proto float asin(float number)
    Returns the arc sine of the number in radians */
 
 PHP_FUNCTION(asin)
@@ -220,7 +233,7 @@ PHP_FUNCTION(asin)
 }
 
 /* }}} */
-/* {{{ proto double acos(double number)
+/* {{{ proto float acos(float number)
    Return the arc cosine of the number in radians */
 
 PHP_FUNCTION(acos)
@@ -236,7 +249,7 @@ PHP_FUNCTION(acos)
 }
 
 /* }}} */
-/* {{{ proto double atan(double number)
+/* {{{ proto float atan(float number)
    Returns the arc tangent of the number in radians */
 
 PHP_FUNCTION(atan)
@@ -252,7 +265,7 @@ PHP_FUNCTION(atan)
 }
 
 /* }}} */
-/* {{{ proto double atan2(double y, double x)
+/* {{{ proto float atan2(float y, float x)
    Returns the arc tangent of y/x, with the resulting quadrant determined by the signs of y and x */
 
 PHP_FUNCTION(atan2)
@@ -269,9 +282,8 @@ PHP_FUNCTION(atan2)
 }
 
 /* }}} */
-/* {{{ proto double sinh(double number)
-   Returns the hyperbolic sine of the number,
-   defined as (exp(number) - exp(-number))/2 */
+/* {{{ proto float sinh(float number)
+   Returns the hyperbolic sine of the number, defined as (exp(number) - exp(-number))/2 */
 
 PHP_FUNCTION(sinh)
 {
@@ -286,9 +298,8 @@ PHP_FUNCTION(sinh)
 }
 
 /* }}} */
-/* {{{ proto double cosh(double number)
-   Returns the hyperbolic cosine of the number,
-   defined as (exp(number) + exp(-number))/2 */
+/* {{{ proto float cosh(float number)
+   Returns the hyperbolic cosine of the number, defined as (exp(number) + exp(-number))/2 */
 
 PHP_FUNCTION(cosh)
 {
@@ -302,9 +313,8 @@ PHP_FUNCTION(cosh)
 	Z_TYPE_P(return_value) = IS_DOUBLE;
 }
 /* }}} */
-/* {{{ proto double tanh(double number)
-   Returns the hyperbolic tangent of the number,
-   defined as sinh(number)/cosh(number) */
+/* {{{ proto float tanh(float number)
+   Returns the hyperbolic tangent of the number, defined as sinh(number)/cosh(number) */
 PHP_FUNCTION(tanh)
 {
 	zval **num;
@@ -320,10 +330,9 @@ PHP_FUNCTION(tanh)
 /* }}} */
 
 #ifndef PHP_WIN32
-/* {{{ proto double asinh(double number)
-   Returns the inverse hyperbolic sine of the number,
-   i.e. the value whose hyperbolic sine is number */
-
+#ifdef HAVE_ASINH
+/* {{{ proto float asinh(float number)
+   Returns the inverse hyperbolic sine of the number, i.e. the value whose hyperbolic sine is number */
 PHP_FUNCTION(asinh)
 {
 	zval **num;
@@ -335,12 +344,12 @@ PHP_FUNCTION(asinh)
 	Z_DVAL_P(return_value) = asinh(Z_DVAL_PP(num));
 	Z_TYPE_P(return_value) = IS_DOUBLE;
 }
-
 /* }}} */
-/* {{{ proto double acosh(double number)
-   Returns the inverse hyperbolic cosine of the number,
-   i.e. the value whose hyperbolic cosine is number */
+#endif /* HAVE_ASINH */
 
+#ifdef HAVE_ACOSH
+/* {{{ proto float acosh(float number)
+   Returns the inverse hyperbolic cosine of the number, i.e. the value whose hyperbolic cosine is number */
 PHP_FUNCTION(acosh)
 {
 	zval **num;
@@ -352,12 +361,12 @@ PHP_FUNCTION(acosh)
 	Z_DVAL_P(return_value) = acosh(Z_DVAL_PP(num));
 	Z_TYPE_P(return_value) = IS_DOUBLE;
 }
-
 /* }}} */
-/* {{{ proto double atanh(double number)
-   Returns the inverse hyperbolic tangent of the number,
-   i.e. the value whose hyperbolic tangent is number */
+#endif /* HAVE_ACOSH */
 
+#ifdef HAVE_ATANH
+/* {{{ proto float atanh(float number)
+   Returns the inverse hyperbolic tangent of the number, i.e. the value whose hyperbolic tangent is number */
 PHP_FUNCTION(atanh)
 {
 	zval **num;
@@ -369,133 +378,112 @@ PHP_FUNCTION(atanh)
 	Z_DVAL_P(return_value) = atanh(Z_DVAL_PP(num));
 	Z_TYPE_P(return_value) = IS_DOUBLE;
 }
-
 /* }}} */
-#endif
+#endif /* HAVE_ATANH */
+#endif /* ifndf PHP_WIN32 */
 
-/* {{{ proto double pi(void)
+
+/* {{{ proto float pi(void)
    Returns an approximation of pi */
-
 PHP_FUNCTION(pi)
 {
 	Z_DVAL_P(return_value) = M_PI;
 	Z_TYPE_P(return_value) = IS_DOUBLE;
 }
+/* }}} */
 
+
+/* {{{ proto bool is_finite(float val)
+   Returns whether argument is finite */
+PHP_FUNCTION(is_finite)
+{
+	double dval;
+
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "d", &dval) == FAILURE) {
+		return;
+	}
+	RETURN_BOOL(zend_finite(dval));
+}
+/* }}} */
+
+/* {{{ proto bool is_infinite(float val)
+   Returns whether argument is infinite */
+PHP_FUNCTION(is_infinite)
+{
+	double dval;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "d", &dval) == FAILURE) {
+		return;
+	}
+	RETURN_BOOL(zend_isinf(dval));
+}
+/* }}} */
+
+/* {{{ proto bool is_nan(float val)
+   Returns whether argument is not a number */
+PHP_FUNCTION(is_nan)
+{
+	double dval;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "d", &dval) == FAILURE) {
+		return;
+	}
+	RETURN_BOOL(zend_isnan(dval));
+}
 /* }}} */
 
 /* {{{ proto number pow(number base, number exponent)
-   Returns base raised to the power of exponent. Returns
-   integer result when possible. */
-
+   Returns base raised to the power of exponent. Returns integer result when possible */
 PHP_FUNCTION(pow)
 {
-	/* FIXME: What is our policy on float-overflow? With pow, it's 
-	 * extremely easy to request results that won't fit in any double.
-	 */
-	
-	zval **zbase, **zexp;
-	long lbase, lexp;
-	double dval;
-	
-	if (ZEND_NUM_ARGS() != 2) {
-		WRONG_PARAM_COUNT;
-	} 
-	zend_get_parameters_ex(ZEND_NUM_ARGS(), &zbase, &zexp);
-	convert_scalar_to_number_ex(zbase);
-	convert_scalar_to_number_ex(zexp);
-	if ((Z_TYPE_PP(zbase) != IS_LONG && Z_TYPE_PP(zbase) != IS_DOUBLE) ||
-		(Z_TYPE_PP(zexp ) != IS_LONG && Z_TYPE_PP(zexp ) != IS_DOUBLE)) {
-		php_error(E_WARNING, "Invalid argument(s) passed to pow()");
-		RETURN_FALSE;
-	}
-	
-	if (Z_TYPE_PP(zexp) == IS_DOUBLE) {
-		/* pow(?, float), this is the ^^ case */
-		convert_to_double_ex(zbase);
+	zval *zbase, *zexp;
 
-		if (Z_DVAL_PP(zbase) < 0.0) {
-			/* Note that with the old behaviour, php pow() returned bogus
-			   results. Try pow(-1, 2.5) in PHP <= 4.0.6 ... */
-			php_error(E_WARNING, "Trying to raise a nonpositive value to a broken power");
-			RETURN_FALSE;
-		}
-		RETURN_DOUBLE(exp(log(Z_DVAL_PP(zbase)) * Z_DVAL_PP(zexp)));
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z/z/", &zbase, &zexp) == FAILURE) {
+		return;
 	}
 
-	/* pow(?, int), this is the ** case */
+	/* make sure we're dealing with numbers */
+	convert_scalar_to_number(zbase TSRMLS_CC);
+	convert_scalar_to_number(zexp TSRMLS_CC);
 
-	lexp = Z_LVAL_PP(zexp);
+	/* if both base and exponent were longs, we'll try to get a long out */
+	if (Z_TYPE_P(zbase) == IS_LONG && Z_TYPE_P(zexp) == IS_LONG && Z_LVAL_P(zexp) >= 0) {
+		long l1 = 1, l2 = Z_LVAL_P(zbase), i = Z_LVAL_P(zexp);
 
-
-	if (Z_TYPE_PP(zbase) == IS_DOUBLE) {
-		/* pow(float, int) */
-		if (lexp == 0) {
-			RETURN_DOUBLE(1.0);
+		if (i == 0) {
+			RETURN_LONG(1L);
+		} else if (l2 == 0) {
+			RETURN_LONG(0);
 		}
-		if (Z_DVAL_PP(zbase) > 0.0) {
-			RETURN_DOUBLE(exp(log(Z_DVAL_PP(zbase)) * lexp));
-		} else if (Z_DVAL_PP(zbase) == 0.0) {
-			if (lexp < 0) {
-				php_error(E_WARNING,
-					"Division by zero: pow(0.0, [negative integer])");
-				RETURN_FALSE;
+
+		/* calculate pow(long,long) in O(log exp) operations, bail if overflow */
+		while (i >= 1) {
+			int overflow;
+			double dval;
+
+			if (i % 2) {
+				--i;
+				ZEND_SIGNED_MULTIPLY_LONG(l1,l2,l1,dval,overflow);
+				if (overflow) RETURN_DOUBLE(dval * pow(l2,i));
 			} else {
-				RETURN_DOUBLE(0.0);
+				i /= 2;
+				ZEND_SIGNED_MULTIPLY_LONG(l2,l2,l2,dval,overflow);
+				if (overflow) RETURN_DOUBLE((double)l1 * pow(dval,i));
+			}
+			if (i == 0) {
+				RETURN_LONG(l1);
 			}
-		} else { /* lbase < 0.0 */
-			dval = exp(log(-Z_DVAL_PP(zbase)) * (double)lexp);
-			RETURN_DOUBLE(lexp & 1 ? -dval : dval);
 		}
-			
-	}
-	
-	/* pow(int, int) */
-	if (lexp == 0) {
-		RETURN_LONG(1);
 	}
+	convert_to_double(zbase);
+	convert_to_double(zexp);
 
-	lbase = Z_LVAL_PP(zbase);
-
-	/* lexp != 0 */
-	switch (lbase) {
-		case -1:
-			RETURN_LONG( lexp & 1 ? -1 : 1 ); /* if lexp=odd ... */
-		case 0:
-			if (lexp < 0) {
-				php_error(E_WARNING,
-					"Division by zero: pow(0, [negative integer])");
-				RETURN_FALSE;
-			} else {
-				RETURN_LONG(0);
-			}
-		case 1:
-			RETURN_LONG(1);
-		default:
-			/* abs(lbase) > 1 */
-			dval = exp(log(lbase>0? (double)lbase : -(double)lbase ) * 
-								  (double) lexp);
-			if (lexp < 0 || dval > (double) LONG_MAX) {
-				/* 1/n ( abs(n) > 1 ) || overflow */
-				RETURN_DOUBLE(((lexp & 1) && lbase<0) ? -dval : dval);
-			}
-
-			Z_TYPE_P(return_value) = IS_LONG;
-			Z_LVAL_P(return_value) = 1;
-
-			/* loop runs at most log(log(LONG_MAX)) times, i.e. ~ 5 */
-			while (lexp > 0) { 
-				if (lexp & 1) /* odd */
-					Z_LVAL_P(return_value) *= lbase;
-				lexp >>= 1;
-				lbase *= lbase;
-			}
-			/* return */
-	}
+	RETURN_DOUBLE( pow(Z_DVAL_P(zbase),Z_DVAL_P(zexp)) );
 }
-
 /* }}} */
-/* {{{ proto double exp(double number)
+
+/* {{{ proto float exp(float number)
    Returns e raised to the power of the number */
 
 PHP_FUNCTION(exp)
@@ -513,13 +501,14 @@ PHP_FUNCTION(exp)
 /* }}} */
 
 
-#ifndef PHP_WIN32
-/* {{{ proto double expm1(double number)
-   Returns exp(number) - 1, computed in a way that accurate even when 
-   the value of number is close to zero 
-   WARNING: this function is expermental: it could change its name or 
+#if !defined(PHP_WIN32) && !defined(NETWARE)
+/* {{{ proto float expm1(float number)
+   Returns exp(number) - 1, computed in a way that accurate even when the value of number is close to zero */
+
+/*
+   WARNING: this function is expermental: it could change its name or
    disappear in the next version of PHP!
-   */
+*/
 
 PHP_FUNCTION(expm1)
 {
@@ -534,15 +523,17 @@ PHP_FUNCTION(expm1)
 }
 
 /* }}} */
-/* {{{ proto double log1p(double number)
-   Returns log(1 + number), computed in a way that accurate even when 
-   the value of number is close to zero 
-   WARNING: this function is expermental: it could change its name or 
+/* {{{ proto float log1p(float number)
+   Returns log(1 + number), computed in a way that accurate even when the value of number is close to zero */
+
+/*
+   WARNING: this function is expermental: it could change its name or
    disappear in the next version of PHP!
-   */
+*/
 
 PHP_FUNCTION(log1p)
 {
+#ifdef HAVE_LOG1P
 	zval **num;
 
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &num) == FAILURE) {
@@ -551,28 +542,45 @@ PHP_FUNCTION(log1p)
 	convert_to_double_ex(num);
 	Z_DVAL_P(return_value) = log1p(Z_DVAL_PP(num));
 	Z_TYPE_P(return_value) = IS_DOUBLE;
+#endif
 }
 
 /* }}} */
 
 #endif
-/* {{{ proto double log(double number)
-   Returns the natural logarithm of the number */
+/* {{{ proto float log(float number, [float base])
+   Returns the natural logarithm of the number, or the base log if base is specified */
 
 PHP_FUNCTION(log)
 {
-	zval **num;
+	zval **num, **base;
 
-	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &num) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	switch (ZEND_NUM_ARGS()) {
+		case 1:
+			if (zend_get_parameters_ex(1, &num) == FAILURE) {
+				WRONG_PARAM_COUNT;
+			}
+			convert_to_double_ex(num);
+			RETURN_DOUBLE(log(Z_DVAL_PP(num)));
+		case 2:
+			if (zend_get_parameters_ex(2, &num, &base) == FAILURE) {
+				WRONG_PARAM_COUNT;
+			}
+			convert_to_double_ex(num);
+			convert_to_double_ex(base);
+
+			if (Z_DVAL_PP(base) <= 0.0) {
+				php_error_docref(NULL TSRMLS_CC, E_WARNING, "base must be greater than 0");
+				RETURN_FALSE;
+			}
+			RETURN_DOUBLE(log(Z_DVAL_PP(num)) / log(Z_DVAL_PP(base)));
+		default:
+			WRONG_PARAM_COUNT;
 	}
-	convert_to_double_ex(num);
-	Z_DVAL_P(return_value) = log(Z_DVAL_PP(num));
-	Z_TYPE_P(return_value) = IS_DOUBLE;
 }
 
 /* }}} */
-/* {{{ proto double log10(double number)
+/* {{{ proto float log10(float number)
    Returns the base-10 logarithm of the number */
 
 PHP_FUNCTION(log10)
@@ -588,7 +596,7 @@ PHP_FUNCTION(log10)
 }
 
 /* }}} */
-/* {{{ proto double sqrt(double number)
+/* {{{ proto float sqrt(float number)
    Returns the square root of the number */
 
 PHP_FUNCTION(sqrt)
@@ -606,12 +614,10 @@ PHP_FUNCTION(sqrt)
 /* }}} */
 
 
-/* {{{ proto double hypot(double num1, double num2)
-   Returns sqrt( num1*num1 + num2*num2) 
-   WARNING: this function is expermental: it could change its name or 
-   disappear in the next version of PHP!
-   */
+/* {{{ proto float hypot(float num1, float num2)
+   Returns sqrt(num1*num1 + num2*num2) */
 
+#ifdef HAVE_HYPOT
 PHP_FUNCTION(hypot)
 {
 	zval **num1, **num2;
@@ -621,13 +627,14 @@ PHP_FUNCTION(hypot)
 	}
 	convert_to_double_ex(num1);
 	convert_to_double_ex(num2);
-	Z_DVAL_P(return_value) = hypot((*num1)->value.dval, (*num2)->value.dval);
+	Z_DVAL_P(return_value) = hypot(Z_DVAL_PP(num1), Z_DVAL_PP(num2));
 	Z_TYPE_P(return_value) = IS_DOUBLE;
 }
+#endif
 
 /* }}} */
 
-/* {{{ proto double deg2rad(double number)
+/* {{{ proto float deg2rad(float number)
    Converts the number in degrees to the radian equivalent */
 
 PHP_FUNCTION(deg2rad)
@@ -642,7 +649,7 @@ PHP_FUNCTION(deg2rad)
 }
 
 /* }}} */
-/* {{{ proto double rad2deg(double number)
+/* {{{ proto float rad2deg(float number)
    Converts the radian number to the equivalent number in degrees */
 
 PHP_FUNCTION(rad2deg)
@@ -664,7 +671,7 @@ PHP_FUNCTION(rad2deg)
  */
 PHPAPI long
 _php_math_basetolong(zval *arg, int base) {
-	long mult = 1, num = 0, digit;
+	long num = 0, digit, onum;
 	int i;
 	char c, *s;
 
@@ -674,23 +681,29 @@ _php_math_basetolong(zval *arg, int base) {
 
 	s = Z_STRVAL_P(arg);
 
-	for (i = Z_STRLEN_P(arg) - 1; i >= 0; i--, mult *= base) {
-		c = toupper(s[i]);
-		if (c >= '0' && c <= '9') {
-			digit = (c - '0');
-		} else if (c >= 'A' && c <= 'Z') {
-			digit = (c - 'A' + 10);
-		} else {
-			continue;
-		}
+	for (i = Z_STRLEN_P(arg); i > 0; i--) {
+		c = *s++;
+
+		digit = (c >= '0' && c <= '9') ? c - '0'
+			: (c >= 'A' && c <= 'Z') ? c - 'A' + 10
+			: (c >= 'a' && c <= 'z') ? c - 'a' + 10
+			: base;
+
 		if (digit >= base) {
 			continue;
 		}
-		if(!mult || digit > LONG_MAX/mult || num > LONG_MAX-mult*digit) {
-			php_error(E_WARNING, "base_to_long: number '%s' is too big to fit in long", s);
+
+		onum = num;
+		num = num * base + digit;
+		if (num > onum)
+			continue;
+
+		{
+			TSRMLS_FETCH();
+
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Number '%s' is too big to fit in long", s);
 			return LONG_MAX;
 		}
-		num += mult * digit;
 	}
 
 	return num;
@@ -706,11 +719,13 @@ _php_math_basetolong(zval *arg, int base) {
  */
 PHPAPI int
 _php_math_basetozval(zval *arg, int base, zval *ret) {
-	long mult = 1, num = 0, digit;
-	double fmult, fnum;
+	long num = 0;
+	double fnum = 0;
 	int i;
-	int f_mode = 0;
+	int mode = 0;
 	char c, *s;
+	long cutoff;
+	int cutlim;
 
 	if (Z_TYPE_P(arg) != IS_STRING || base < 2 || base > 36) {
 		return FAILURE;
@@ -718,38 +733,41 @@ _php_math_basetozval(zval *arg, int base, zval *ret) {
 
 	s = Z_STRVAL_P(arg);
 
-	for (i = Z_STRLEN_P(arg) - 1; i >= 0; i--) {
-		c = toupper(s[i]);
-		if (c >= '0' && c <= '9') {
-			digit = (c - '0');
-		} else if (c >= 'A' && c <= 'Z') {
-			digit = (c - 'A' + 10);
-		} else {
+	cutoff = LONG_MAX / base;
+	cutlim = LONG_MAX % base;
+
+	for (i = Z_STRLEN_P(arg); i > 0; i--) {
+		c = *s++;
+
+		/* might not work for EBCDIC */
+		if (c >= '0' && c <= '9')
+			c -= '0';
+		else if (c >= 'A' && c <= 'Z')
+			c -= 'A' - 10;
+		else if (c >= 'a' && c <= 'z')
+			c -= 'a' - 10;
+		else
 			continue;
-		}
-		if (digit >= base) {
+
+		if (c >= base)
 			continue;
-		}
-		if(!f_mode && (!mult || digit > LONG_MAX/mult || num > LONG_MAX-mult*digit)) {
-			f_mode = 1;
-			if(!mult) {
-				fmult = ((double) ULONG_MAX) + 1;
+
+		switch (mode) {
+		case 0: /* Integer */
+			if (num < cutoff || (num == cutoff && c <= cutlim)) {
+				num = num * base + c;
+				break;
 			} else {
-				fmult = (unsigned long)mult;
+				fnum = num;
+				mode = 1;
 			}
-			fnum = (unsigned long)num;
-		}
-		
-		if(f_mode) {
-			fnum += fmult * digit;
-			fmult *= base;
-		} else {
-			num += mult * digit;
-			mult *= base;
+			/* fall-through */
+		case 1: /* Float */
+			fnum = fnum * base + c;
 		}
 	}
 
-	if(f_mode) {
+	if (mode == 1) {
 		ZVAL_DOUBLE(ret, fnum);
 	} else {
 		ZVAL_LONG(ret, num);
@@ -768,8 +786,8 @@ PHPAPI char *
 _php_math_longtobase(zval *arg, int base)
 {
 	static char digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";
-	char *result, *ptr, *ret;
-	int len, digit;
+	char buf[(sizeof(unsigned long) << 3) + 1];
+	char *ptr, *end;
 	unsigned long value;
 
 	if (Z_TYPE_P(arg) != IS_LONG || base < 2 || base > 36) {
@@ -778,25 +796,16 @@ _php_math_longtobase(zval *arg, int base)
 
 	value = Z_LVAL_P(arg);
 
-	/* allocates space for the longest possible result with the lowest base */
-	len = (sizeof(Z_LVAL_P(arg)) * 8) + 1;
-	result = emalloc((sizeof(Z_LVAL_P(arg)) * 8) + 1);
-
-	ptr = result + len - 1;
-	*ptr-- = '\0';
+	end = ptr = buf + sizeof(buf) - 1;
+	*ptr = '\0';
 
 	do {
-		digit = value % base;
-		*ptr = digits[digit];
+		*--ptr = digits[value % base];
 		value /= base;
-	}
-	while (ptr-- > result && value);
-	ptr++;
-	ret = estrdup(ptr);
-	efree(result);
+	} while (ptr > buf && value);
 
-	return ret;
-}	
+	return estrndup(ptr, end - ptr);
+}
 
 /* }}} */
 /* {{{ _php_math_zvaltobase */
@@ -806,53 +815,38 @@ _php_math_longtobase(zval *arg, int base)
  * the number.
  */
 PHPAPI char *
-_php_math_zvaltobase(zval *arg, int base)
+_php_math_zvaltobase(zval *arg, int base TSRMLS_DC)
 {
 	static char digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";
-	char *result, *ptr, *ret;
-	int len, digit;
-	unsigned long value;
-	double fvalue;
-	int f_mode;
 
 	if ((Z_TYPE_P(arg) != IS_LONG && Z_TYPE_P(arg) != IS_DOUBLE) || base < 2 || base > 36) {
 		return empty_string;
 	}
 
-	f_mode = (Z_TYPE_P(arg) == IS_DOUBLE);
+	if (Z_TYPE_P(arg) == IS_DOUBLE) {
+		double fvalue = floor(Z_DVAL_P(arg)); /* floor it just in case */
+		char *ptr, *end;
+		char buf[(sizeof(double) << 3) + 1];
 
-	if(f_mode) {
-		fvalue = floor(Z_DVAL_P(arg)); /* floor it just in case */
-	} else {
-		value = Z_LVAL_P(arg);
-	}
-
-	/* allocates space for the longest possible result with the lowest base */
-	len = (sizeof(Z_DVAL_P(arg)) * 8) + 1;
-	result = emalloc((sizeof(Z_DVAL_P(arg)) * 8) + 1);
-
-	ptr = result + len - 1;
-	*ptr-- = '\0';
-
-	do {
-		if(f_mode) {
-			double d = floor(fvalue/base);
-			digit = (int)ceil(fvalue - d*base);
-			*ptr = digits[digit];
-			fvalue = d;
-		} else {
-			digit = value % base;
-			*ptr = digits[digit];
-			value /= base;
+		/* Don't try to convert +/- infinity */
+		if (fvalue == HUGE_VAL || fvalue == -HUGE_VAL) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Number too large");
+			return empty_string;
 		}
-	}
-	while (ptr-- > result && (f_mode?(fabs(fvalue)>=1):value));
-	ptr++;
-	ret = estrdup(ptr);
-	efree(result);
 
-	return ret;
-}	
+		end = ptr = buf + sizeof(buf) - 1;
+		*ptr = '\0';
+
+		do {
+			*--ptr = digits[(int) fmod(fvalue, base)];
+			fvalue /= base;
+		} while (ptr > buf && fabs(fvalue) >= 1);
+
+		return estrndup(ptr, end - ptr);
+	}
+
+	return _php_math_longtobase(arg, base);
+}
 
 /* }}} */
 /* {{{ proto int bindec(string binary_number)
@@ -861,7 +855,7 @@ _php_math_zvaltobase(zval *arg, int base)
 PHP_FUNCTION(bindec)
 {
 	zval **arg;
-	
+
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg) == FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
@@ -879,7 +873,7 @@ PHP_FUNCTION(bindec)
 PHP_FUNCTION(hexdec)
 {
 	zval **arg;
-	
+
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg) == FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
@@ -898,7 +892,7 @@ PHP_FUNCTION(hexdec)
 PHP_FUNCTION(octdec)
 {
 	zval **arg;
-	
+
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg) == FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
@@ -975,9 +969,7 @@ PHP_FUNCTION(dechex)
 
 /* }}} */
 /* {{{ proto string base_convert(string number, int frombase, int tobase)
-
-   Converts a number in a string from any base <= 36 to any base <= 36.
-*/
+   Converts a number in a string from any base <= 36 to any base <= 36 */
 
 PHP_FUNCTION(base_convert)
 {
@@ -991,89 +983,137 @@ PHP_FUNCTION(base_convert)
 	convert_to_long_ex(frombase);
 	convert_to_long_ex(tobase);
 	if (Z_LVAL_PP(frombase) < 2 || Z_LVAL_PP(frombase) > 36) {
-		php_error(E_WARNING, "base_convert: invalid `from base' (%d)", Z_LVAL_PP(frombase));
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid `from base' (%ld)", Z_LVAL_PP(frombase));
 		RETURN_FALSE;
 	}
 	if (Z_LVAL_PP(tobase) < 2 || Z_LVAL_PP(tobase) > 36) {
-		php_error(E_WARNING, "base_convert: invalid `to base' (%d)", Z_LVAL_PP(tobase));
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid `to base' (%ld)", Z_LVAL_PP(tobase));
 		RETURN_FALSE;
 	}
 
 	if(_php_math_basetozval(*number, Z_LVAL_PP(frombase), &temp) != SUCCESS) {
 		RETURN_FALSE;
 	}
-	result = _php_math_zvaltobase(&temp, Z_LVAL_PP(tobase));
+	result = _php_math_zvaltobase(&temp, Z_LVAL_PP(tobase) TSRMLS_CC);
 	RETVAL_STRING(result, 0);
-} 
+}
 
 /* }}} */
 /* {{{ _php_math_number_format */
 
-char *_php_math_number_format(double d, int dec, char dec_point, char thousand_sep)
+PHPAPI char *_php_math_number_format(double d, int dec, char dec_point, char thousand_sep)
 {
-	char *tmpbuf, *resbuf;
+	char *tmpbuf = NULL, *resbuf;
 	char *s, *t;  /* source, target */
+	char *dp;
+	int integral;
 	int tmplen, reslen=0;
 	int count=0;
 	int is_negative=0;
-	
-	if (d<0) {
-		is_negative=1;
+
+	if (d < 0) {
+		is_negative = 1;
 		d = -d;
 	}
-	dec = MAX(0, dec);
-	tmpbuf = (char *) emalloc(1+DBL_MAX_10_EXP+1+dec+1);
-	
-	tmplen=sprintf(tmpbuf, "%.*f", dec, d);
+	if (!dec_point && dec > 0) {
+		d *= pow(10, dec);
+		dec = 0;
+	} else {
+		dec = MAX(0, dec);
+	}
 
-	if (!isdigit((int)tmpbuf[0])) {
+	PHP_ROUND_WITH_FUZZ(d, dec);
+
+	tmplen = spprintf(&tmpbuf, 0, "%.*f", dec, d);
+
+	if (tmpbuf == NULL || !isdigit((int)tmpbuf[0])) {
 		return tmpbuf;
 	}
 
-	if (dec) {
-		reslen = dec+1 + (tmplen-dec-1) + ((thousand_sep) ? (tmplen-1-dec-1)/3 : 0);
+	/* calculate the length of the return buffer */
+	dp = strchr(tmpbuf, '.');
+	if (dp) {
+		integral = dp - tmpbuf;
 	} else {
-		reslen = tmplen+((thousand_sep) ? (tmplen-1)/3 : 0);
+		/* no decimal point was found */
+		integral = tmplen;
 	}
+
+	/* allow for thousand separators */
+	if (thousand_sep) {
+		integral += (integral-1) / 3;
+	}
+
+	reslen = integral;
+
+	if (dec) {
+		reslen += 1 + dec;
+	}
+
+	/* add a byte for minus sign */
 	if (is_negative) {
 		reslen++;
 	}
-	resbuf = (char *) emalloc(reslen+1);
-	
+	resbuf = (char *) emalloc(reslen+1); /* +1 for NUL terminator */
+
 	s = tmpbuf+tmplen-1;
 	t = resbuf+reslen;
-	*t-- = 0;
-	
+	*t-- = '\0';
+
+	/* copy the decimal places.
+	 * Take care, as the sprintf implementation may return less places than
+	 * we requested due to internal buffer limitations */
 	if (dec) {
-		while (isdigit((int)*s)) {
-			*t-- = *s--;
+		int declen = dp ? strlen(dp+1) : 0;
+		int topad = declen > 0 ? dec - declen : 0;
+
+		/* pad with '0's */
+
+		while (topad--) {
+			*t-- = '0';
 		}
-		*t-- = dec_point;  /* copy that dot */
+
+		if (dp) {
+			/* now copy the chars after the point */
+			memcpy(t - declen + 1, dp + 1, declen);
+
+			t -= declen;
+			s -= declen;
+		}
+
+		/* add decimal point */
+		*t-- = dec_point;
 		s--;
 	}
-	
-	while(s>=tmpbuf) {
+
+	/* copy the numbers before the decimal place, adding thousand
+	 * separator every three digits */
+	while(s >= tmpbuf) {
 		*t-- = *s--;
 		if (thousand_sep && (++count%3)==0 && s>=tmpbuf) {
 			*t-- = thousand_sep;
 		}
 	}
+
+	/* and a minus sign, if needed */
 	if (is_negative) {
 		*t-- = '-';
 	}
+
 	efree(tmpbuf);
+
 	return resbuf;
 }
 
 /* }}} */
-/* {{{ proto string number_format(double number [, int num_decimal_places [, string dec_seperator, string thousands_seperator]])
+/* {{{ proto string number_format(float number [, int num_decimal_places [, string dec_seperator, string thousands_seperator]])
    Formats a number with grouped thousands */
 
 PHP_FUNCTION(number_format)
 {
 	zval **num, **dec, **t_s, **d_p;
 	char thousand_sep=',', dec_point='.';
-	
+
 	switch(ZEND_NUM_ARGS()) {
 	case 1:
 		if (zend_get_parameters_ex(1, &num)==FAILURE) {
@@ -1096,15 +1136,22 @@ PHP_FUNCTION(number_format)
 		}
 		convert_to_double_ex(num);
 		convert_to_long_ex(dec);
-		convert_to_string_ex(d_p);
-		convert_to_string_ex(t_s);
-		if (Z_STRLEN_PP(d_p)==1) {
-			dec_point=Z_STRVAL_PP(d_p)[0];
+
+		if (Z_TYPE_PP(d_p) != IS_NULL) {
+			convert_to_string_ex(d_p);
+			if (Z_STRLEN_PP(d_p)>=1) {
+				dec_point=Z_STRVAL_PP(d_p)[0];
+			} else if (Z_STRLEN_PP(d_p)==0) {
+				dec_point=0;
+			}
 		}
-		if (Z_STRLEN_PP(t_s)==1) {
-			thousand_sep=Z_STRVAL_PP(t_s)[0];
-		} else if(Z_STRLEN_PP(t_s)==0) {
-			thousand_sep=0;	
+		if (Z_TYPE_PP(t_s) != IS_NULL) {
+			convert_to_string_ex(t_s);
+			if (Z_STRLEN_PP(t_s)>=1) {
+				thousand_sep=Z_STRVAL_PP(t_s)[0];
+			} else if(Z_STRLEN_PP(t_s)==0) {
+				thousand_sep=0;
+			}
 		}
 		RETURN_STRING(_php_math_number_format(Z_DVAL_PP(num), Z_LVAL_PP(dec), dec_point, thousand_sep), 0);
 		break;
@@ -1115,11 +1162,26 @@ PHP_FUNCTION(number_format)
 }
 /* }}} */
 
+/* {{{ proto float fmod(float x, float y)
+   Returns the remainder of dividing x by y as a float */
+PHP_FUNCTION(fmod)
+{
+	double num1, num2;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "dd",  &num1, &num2) == FAILURE) {
+		return;
+	}
+
+	Z_DVAL_P(return_value) = fmod(num1, num2);
+	Z_TYPE_P(return_value) = IS_DOUBLE;
+}
+/* }}} */
+
 /*
  * Local variables:
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: sw=4 ts=4 tw=78 fdm=marker
- * vim<600: sw=4 ts=4 tw=78
+ * vim600: fdm=marker
+ * vim: noet sw=4 ts=4
  */
