@@ -1,22 +1,22 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP version 4.0                                                      |
+   | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2001 The PHP Group                                |
+   | Copyright (c) 1997-2006 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 2.02 of the PHP license,      |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
-   | http://www.php.net/license/2_02.txt.                                 |
+   | available through the world-wide-web at the following url:           |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Authors: Andrei Zmievski <andrei@ispi.net>                           |
+   | Author: Andrei Zmievski <andrei@php.net>                             |
    +----------------------------------------------------------------------+
  */
 
-/* $Id: wddx.c,v 1.79.2.4 2001/11/18 01:23:14 andrei Exp $ */
+/* $Id: wddx.c,v 1.96.2.6.2.3 2006/01/01 13:46:59 sniper Exp $ */
 
 #include "php.h"
 #include "php_wddx.h"
@@ -31,6 +31,8 @@
 #include "ext/standard/info.h"
 #include "ext/standard/php_smart_str.h"
 #include "ext/standard/html.h"
+#include "ext/standard/php_string.h"
+#include "ext/standard/php_parsedate.h"
 
 #define WDDX_BUF_LEN			256
 #define PHP_CLASS_NAME_VAR		"php_class_name"
@@ -47,8 +49,11 @@
 #define EL_STRUCT				"struct"
 #define EL_VALUE				"value"
 #define EL_VAR					"var"
-#define EL_VAR_NAME				"name"
+#define EL_NAME	    			"name"
 #define EL_VERSION				"version"
+#define EL_RECORDSET			"recordset"
+#define EL_FIELD				"field"
+#define EL_DATETIME				"dateTime"
 
 #define php_wddx_deserialize(a,b) \
 	php_wddx_deserialize_ex((a)->value.str.val, (a)->value.str.len, (b))
@@ -72,7 +77,10 @@ typedef struct {
 		ST_NUMBER,
 		ST_STRING,
 		ST_BINARY,
-		ST_STRUCT
+		ST_STRUCT,
+		ST_RECORDSET,
+		ST_FIELD,
+		ST_DATETIME
 	} type;
 	char *varname;
 } st_entry;
@@ -85,7 +93,7 @@ typedef struct {
 } wddx_stack;
 
 
-static void php_wddx_process_data(void *user_data, const char *s, int len);
+static void php_wddx_process_data(void *user_data, const XML_Char *s, int len);
 
 /* {{{ wddx_functions[]
  */
@@ -103,6 +111,12 @@ function_entry wddx_functions[] = {
 PHP_MINIT_FUNCTION(wddx);
 PHP_MINFO_FUNCTION(wddx);
 
+/* {{{ dynamically loadable module stuff */
+#ifdef COMPILE_DL_WDDX
+ZEND_GET_MODULE(wddx)
+#endif /* COMPILE_DL_WDDX */
+/* }}} */
+
 /* {{{ wddx_module_entry
  */
 zend_module_entry wddx_module_entry = {
@@ -229,7 +243,7 @@ PS_SERIALIZER_ENCODE_FUNC(wddx)
 	php_wddx_add_chunk_static(packet, WDDX_STRUCT_S);
 	
 	PS_ENCODE_LOOP(
-		php_wddx_serialize_var(packet, *struc, key, key_length);
+		php_wddx_serialize_var(packet, *struc, key, key_length TSRMLS_CC);
 	);
 	
 	php_wddx_add_chunk_static(packet, WDDX_STRUCT_E);
@@ -309,9 +323,11 @@ PHP_MINIT_FUNCTION(wddx)
 PHP_MINFO_FUNCTION(wddx)
 {
 	php_info_print_table_start();
-	php_info_print_table_header(2, "WDDX Support", "enabled" );
 #if HAVE_PHP_SESSION
+	php_info_print_table_header(2, "WDDX Support", "enabled" );
 	php_info_print_table_row(2, "WDDX Session Serializer", "enabled" );
+#else
+	php_info_print_table_row(2, "WDDX Support", "enabled" );
 #endif
 	php_info_print_table_end();
 }
@@ -384,7 +400,7 @@ static void php_wddx_serialize_string(wddx_packet *packet, zval *var)
 					break;
 
 				default:
-					if (iscntrl((int)*p) && *p != '\n') {
+					if (iscntrl((int)*(unsigned char *)p)) {
 						FLUSH_BUF();
 						sprintf(control_buf, WDDX_CHAR, *p);
 						php_wddx_add_chunk(packet, control_buf);
@@ -482,7 +498,7 @@ static void php_wddx_serialize_object(wddx_packet *packet, zval *obj)
 				}
 
 				if (zend_hash_find(HASH_OF(obj), Z_STRVAL_PP(varname), Z_STRLEN_PP(varname)+1, (void **)&ent) == SUCCESS) {
-					php_wddx_serialize_var(packet, *ent, Z_STRVAL_PP(varname), Z_STRLEN_PP(varname));
+					php_wddx_serialize_var(packet, *ent, Z_STRVAL_PP(varname), Z_STRLEN_PP(varname) TSRMLS_CC);
 				}
 			}
 			
@@ -512,10 +528,10 @@ static void php_wddx_serialize_object(wddx_packet *packet, zval *obj)
 				continue;
 
 			if (zend_hash_get_current_key_ex(HASH_OF(obj), &key, &key_len, &idx, 0, NULL) == HASH_KEY_IS_STRING) {
-				php_wddx_serialize_var(packet, *ent, key, key_len);
+				php_wddx_serialize_var(packet, *ent, key, key_len TSRMLS_CC);
 			} else {
 				key_len = sprintf(tmp_buf, "%ld", idx);
-				php_wddx_serialize_var(packet, *ent, tmp_buf, key_len);
+				php_wddx_serialize_var(packet, *ent, tmp_buf, key_len TSRMLS_CC);
 			}
 		}
 		php_wddx_add_chunk_static(packet, WDDX_STRUCT_E);
@@ -543,6 +559,7 @@ static void php_wddx_serialize_array(wddx_packet *packet, zval *arr)
 	char tmp_buf[WDDX_BUF_LEN];
 	ulong ind = 0;
 	int type;
+	TSRMLS_FETCH();
 
 	target_hash = HASH_OF(arr);
 
@@ -582,13 +599,13 @@ static void php_wddx_serialize_array(wddx_packet *packet, zval *arr)
 			ent_type = zend_hash_get_current_key_ex(target_hash, &key, &key_len, &idx, 0, NULL);
 
 			if (ent_type == HASH_KEY_IS_STRING) {
-				php_wddx_serialize_var(packet, *ent, key, key_len);
+				php_wddx_serialize_var(packet, *ent, key, key_len TSRMLS_CC);
 			} else {
 				key_len = sprintf(tmp_buf, "%ld", idx);
-				php_wddx_serialize_var(packet, *ent, tmp_buf, key_len);
+				php_wddx_serialize_var(packet, *ent, tmp_buf, key_len TSRMLS_CC);
 			}
 		} else
-			php_wddx_serialize_var(packet, *ent, NULL, 0);
+			php_wddx_serialize_var(packet, *ent, NULL, 0 TSRMLS_CC);
 	}
 	
 	if (is_struct) {
@@ -601,14 +618,14 @@ static void php_wddx_serialize_array(wddx_packet *packet, zval *arr)
 
 /* {{{ php_wddx_serialize_var
  */
-void php_wddx_serialize_var(wddx_packet *packet, zval *var, char *name, int name_len)
+void php_wddx_serialize_var(wddx_packet *packet, zval *var, char *name, int name_len TSRMLS_DC)
 {
 	char tmp_buf[WDDX_BUF_LEN];
 	char *name_esc;
 	int name_esc_len;
 
 	if (name) {
-		name_esc = php_escape_html_entities(name, name_len, &name_esc_len, 0, ENT_QUOTES, NULL);
+		name_esc = php_escape_html_entities(name, name_len, &name_esc_len, 0, ENT_QUOTES, NULL TSRMLS_CC);
 		sprintf(tmp_buf, WDDX_VAR_S, name_esc);
 		php_wddx_add_chunk(packet, tmp_buf);
 		efree(name_esc);
@@ -659,7 +676,7 @@ static void php_wddx_add_var(wddx_packet *packet, zval *name_var)
 	{
 		if (zend_hash_find(EG(active_symbol_table), Z_STRVAL_P(name_var),
 							Z_STRLEN_P(name_var)+1, (void**)&val) != FAILURE) {
-			php_wddx_serialize_var(packet, *val, Z_STRVAL_P(name_var), Z_STRLEN_P(name_var));
+			php_wddx_serialize_var(packet, *val, Z_STRVAL_P(name_var), Z_STRLEN_P(name_var) TSRMLS_CC);
 		}		
 	}
 	else if (Z_TYPE_P(name_var) == IS_ARRAY || Z_TYPE_P(name_var) == IS_OBJECT)
@@ -679,7 +696,7 @@ static void php_wddx_add_var(wddx_packet *packet, zval *name_var)
 
 /* {{{ php_wddx_push_element
  */
-static void php_wddx_push_element(void *user_data, const char *name, const char **atts)
+static void php_wddx_push_element(void *user_data, const XML_Char *name, const XML_Char **atts)
 {
 	st_entry ent;
 	wddx_stack *stack = (wddx_stack *)user_data;
@@ -714,12 +731,14 @@ static void php_wddx_push_element(void *user_data, const char *name, const char
 		wddx_stack_push((wddx_stack *)stack, &ent, sizeof(st_entry));
 	} else if (!strcmp(name, EL_CHAR)) {
 		int i;
-		char tmp_buf[2];
 		
-		for (i=0; atts[i]; i++) {
-			if (!strcmp(atts[i], EL_CHAR_CODE) && atts[i+1]) {
-				sprintf(tmp_buf, "%c", (char)strtol(atts[i+1], NULL, 16));
+		for (i = 0; atts[i]; i++) {
+			if (!strcmp(atts[i], EL_CHAR_CODE) && atts[++i] && atts[i][0]) {
+				char tmp_buf[2];
+
+				sprintf(tmp_buf, "%c", (char)strtol(atts[i], NULL, 16));
 				php_wddx_process_data(user_data, tmp_buf, strlen(tmp_buf));
+				break;
 			}
 		}
 	} else if (!strcmp(name, EL_NUMBER)) {
@@ -733,8 +752,8 @@ static void php_wddx_push_element(void *user_data, const char *name, const char
 	} else if (!strcmp(name, EL_BOOLEAN)) {
 		int i;
 
-		for (i=0; atts[i]; i++) {
-			if (!strcmp(atts[i], EL_VALUE) && atts[i+1]) {
+		for (i = 0; atts[i]; i++) {
+			if (!strcmp(atts[i], EL_VALUE) && atts[++i] && atts[i][0]) {
 				ent.type = ST_BOOLEAN;
 				SET_STACK_VARNAME;
 
@@ -742,7 +761,8 @@ static void php_wddx_push_element(void *user_data, const char *name, const char
 				INIT_PZVAL(ent.data);
 				Z_TYPE_P(ent.data) = IS_BOOL;
 				wddx_stack_push((wddx_stack *)stack, &ent, sizeof(st_entry));
-				php_wddx_process_data(user_data, atts[i+1], strlen(atts[i+1]));
+				php_wddx_process_data(user_data, atts[i], strlen(atts[i]));
+				break;
 			}
 		}
 	} else if (!strcmp(name, EL_NULL)) {
@@ -773,21 +793,98 @@ static void php_wddx_push_element(void *user_data, const char *name, const char
 	} else if (!strcmp(name, EL_VAR)) {
 		int i;
 		
-		for (i=0; atts[i]; i++) {
-			if (!strcmp(atts[i], EL_VAR_NAME) && atts[i+1]) {
-				char *decoded_value;
+		for (i = 0; atts[i]; i++) {
+			if (!strcmp(atts[i], EL_NAME) && atts[++i] && atts[i][0]) {
+				char *decoded;
 				int decoded_len;
-				decoded_value = xml_utf8_decode(atts[i+1],strlen(atts[i+1]),&decoded_len,"ISO-8859-1");
-				stack->varname = decoded_value;
+				decoded = xml_utf8_decode(atts[i], strlen(atts[i]), &decoded_len, "ISO-8859-1");
+				stack->varname = decoded;
+				break;
 			}
 		}
+	} else if (!strcmp(name, EL_RECORDSET)) {
+		int i;
+
+		ent.type = ST_RECORDSET;
+		SET_STACK_VARNAME;
+		MAKE_STD_ZVAL(ent.data);
+		array_init(ent.data);
+
+		for (i = 0; atts[i]; i++) {
+			if (!strcmp(atts[i], "fieldNames") && atts[++i] && atts[i][0]) {
+				zval *tmp;
+				char *key;
+				char *p1, *p2, *endp;
+				char *decoded;
+				int decoded_len;
+
+				decoded = xml_utf8_decode(atts[i], strlen(atts[i]), &decoded_len, "ISO-8859-1");
+				endp = (char *)decoded + decoded_len;
+				p1 = (char *)decoded;
+				while ((p2 = php_memnstr(p1, ",", sizeof(",")-1, endp)) != NULL) {
+					key = estrndup(p1, p2 - p1);
+					MAKE_STD_ZVAL(tmp);
+					array_init(tmp);
+					add_assoc_zval_ex(ent.data, key, p2 - p1 + 1, tmp);
+					p1 = p2 + sizeof(",")-1;
+					efree(key);
+				}
+
+				if (p1 <= endp) {
+					MAKE_STD_ZVAL(tmp);
+					array_init(tmp);
+					add_assoc_zval_ex(ent.data, p1, endp - p1 + 1, tmp);
+				}
+
+				efree(decoded);
+				break;
+			}
+		}
+
+		wddx_stack_push((wddx_stack *)stack, &ent, sizeof(st_entry));
+	} else if (!strcmp(name, EL_FIELD)) {
+		int i;
+		st_entry ent;
+
+		ent.type = ST_FIELD;
+		ent.varname = NULL;
+		ent.data = NULL;
+
+		for (i = 0; atts[i]; i++) {
+			if (!strcmp(atts[i], EL_NAME) && atts[++i] && atts[i][0]) {
+				char *decoded;
+				int decoded_len;
+				st_entry *recordset;
+				zval **field;
+ 
+				decoded = xml_utf8_decode(atts[i], strlen(atts[i]), &decoded_len, "ISO-8859-1");
+				if (wddx_stack_top(stack, (void**)&recordset) == SUCCESS &&
+					recordset->type == ST_RECORDSET &&
+					zend_hash_find(Z_ARRVAL_P(recordset->data), decoded, decoded_len+1, (void**)&field) == SUCCESS) {
+					ent.data = *field;
+				}
+				
+				efree(decoded);
+				break;
+			}
+		}
+
+		wddx_stack_push((wddx_stack *)stack, &ent, sizeof(st_entry));
+	} else if (!strcmp(name, EL_DATETIME)) {
+		ent.type = ST_DATETIME;
+		SET_STACK_VARNAME;
+		
+		ALLOC_ZVAL(ent.data);
+		INIT_PZVAL(ent.data);
+		Z_TYPE_P(ent.data) = IS_LONG;
+		wddx_stack_push((wddx_stack *)stack, &ent, sizeof(st_entry));
 	}
 }
 /* }}} */
 
 /* {{{ php_wddx_pop_element
  */
-static void php_wddx_pop_element(void *user_data, const char *name)
+static void php_wddx_pop_element(void *user_data, const XML_Char *name)
 {
 	st_entry 			*ent1, *ent2;
 	wddx_stack 			*stack = (wddx_stack *)user_data;
@@ -804,7 +901,8 @@ static void php_wddx_pop_element(void *user_data, const char *name)
 	if (!strcmp(name, EL_STRING) || !strcmp(name, EL_NUMBER) ||
 		!strcmp(name, EL_BOOLEAN) || !strcmp(name, EL_NULL) ||
 	  	!strcmp(name, EL_ARRAY) || !strcmp(name, EL_STRUCT) ||
-		!strcmp(name, EL_BINARY)) {
+		!strcmp(name, EL_RECORDSET) || !strcmp(name, EL_BINARY) ||
+		!strcmp(name, EL_DATETIME)) {
 		wddx_stack_top(stack, (void**)&ent1);
 
 		if (!strcmp(name, EL_BINARY)) {
@@ -835,6 +933,14 @@ static void php_wddx_pop_element(void *user_data, const char *name)
 		if (stack->top > 1) {
 			stack->top--;
 			wddx_stack_top(stack, (void**)&ent2);
+			
+			/* if non-existent field */
+			if (ent2->type == ST_FIELD && ent2->data == NULL) {
+				zval_ptr_dtor(&ent1->data);
+				efree(ent1);
+				return;
+			}
+			
 			if (Z_TYPE_P(ent2->data) == IS_ARRAY || Z_TYPE_P(ent2->data) == IS_OBJECT) {
 				target_hash = HASH_OF(ent2->data);
 
@@ -862,7 +968,7 @@ static void php_wddx_pop_element(void *user_data, const char *name)
 										(void *) &tmp, sizeof(zval *), 0);
 
 						if (incomplete_class) {
-							php_store_class_name(obj, Z_STRVAL_P(ent1->data), Z_STRLEN_P(ent1->data));
+							php_store_class_name(obj, Z_STRVAL_P(ent1->data), Z_STRLEN_P(ent1->data) TSRMLS_CC);
 						}
 
 						/* Clean up old array entry */
@@ -873,11 +979,24 @@ static void php_wddx_pop_element(void *user_data, const char *name)
 						
 						/* Clean up class name var entry */
 						zval_ptr_dtor(&ent1->data);
+					} else {
+						long l;  
+						double d;
+				
+						switch (is_numeric_string(ent1->varname, strlen(ent1->varname), &l, &d, 0)) {
+							case IS_DOUBLE:
+								if (d > INT_MAX) {
+									goto bigint;
+								}
+								l = (long) d;
+							case IS_LONG:
+								zend_hash_index_update(target_hash, l, &ent1->data, sizeof(zval *), NULL);
+								break;
+							default:
+bigint:
+								zend_hash_update(target_hash,ent1->varname, strlen(ent1->varname)+1, &ent1->data, sizeof(zval *), NULL);
+						}
 					}
-					else
-						zend_hash_update(target_hash,
-										 ent1->varname, strlen(ent1->varname)+1,
-										 &ent1->data, sizeof(zval *), NULL);
 					efree(ent1->varname);
 				} else	{
 					zend_hash_next_index_insert(target_hash,
@@ -890,17 +1009,22 @@ static void php_wddx_pop_element(void *user_data, const char *name)
 			stack->done = 1;
 	} else if (!strcmp(name, EL_VAR) && stack->varname) {
 		efree(stack->varname);
+	} else if (!strcmp(name, EL_FIELD)) {
+		st_entry *ent;
+		wddx_stack_top(stack, (void **)&ent);
+		efree(ent);
+		stack->top--;
 	}
 }
 /* }}} */
 
 /* {{{ php_wddx_process_data
  */
-static void php_wddx_process_data(void *user_data, const char *s, int len)
+static void php_wddx_process_data(void *user_data, const XML_Char *s, int len)
 {
 	st_entry *ent;
 	wddx_stack *stack = (wddx_stack *)user_data;
-	char *decoded_value;
+	char *decoded;
 	int decoded_len;
 	TSRMLS_FETCH();
 
@@ -908,20 +1032,20 @@ static void php_wddx_process_data(void *user_data, const char *s, int len)
 		wddx_stack_top(stack, (void**)&ent);
 		switch (Z_TYPE_P(ent)) {
 			case ST_STRING: 
-				decoded_value = xml_utf8_decode(s,len,&decoded_len,"ISO-8859-1");
+				decoded = xml_utf8_decode(s, len, &decoded_len, "ISO-8859-1");
 
 				if (Z_STRLEN_P(ent->data) == 0) {
-					Z_STRVAL_P(ent->data) = estrndup(decoded_value, decoded_len);
+					Z_STRVAL_P(ent->data) = estrndup(decoded, decoded_len);
 					Z_STRLEN_P(ent->data) = decoded_len;
 				} else {
 					Z_STRVAL_P(ent->data) = erealloc(Z_STRVAL_P(ent->data),
 							Z_STRLEN_P(ent->data) + decoded_len + 1);
-					strncpy(Z_STRVAL_P(ent->data)+Z_STRLEN_P(ent->data), decoded_value, decoded_len);
+					strncpy(Z_STRVAL_P(ent->data)+Z_STRLEN_P(ent->data), decoded, decoded_len);
 					Z_STRLEN_P(ent->data) += decoded_len;
 					Z_STRVAL_P(ent->data)[Z_STRLEN_P(ent->data)] = '\0';
 				}
 
-				efree(decoded_value);
+				efree(decoded);
 				break;
 
 			case ST_BINARY:
@@ -956,6 +1080,22 @@ static void php_wddx_process_data(void *user_data, const char *s, int len)
 				}
 				break;
 
+			case ST_DATETIME: {
+				char *tmp;
+
+				tmp = emalloc(len + 1);
+				memcpy(tmp, s, len);
+				tmp[len] = '\0';
+
+				Z_LVAL_P(ent->data) = php_parse_date(tmp, NULL);
+				/* date out of range < 1969 or > 2038 */
+				if (Z_LVAL_P(ent->data) == -1) {
+					Z_TYPE_P(ent->data) = IS_STRING;
+					Z_STRLEN_P(ent->data) = len;
+					Z_STRVAL_P(ent->data) = estrndup(s, len);
+				}
+				efree(tmp);
+			}
 			default:
 				break;
 		}
@@ -1016,7 +1156,7 @@ PHP_FUNCTION(wddx_serialize_value)
 	}
 
 	php_wddx_packet_start(packet, comment, comment_len);
-	php_wddx_serialize_var(packet, var, NULL, 0);
+	php_wddx_serialize_var(packet, var, NULL, 0 TSRMLS_CC);
 	php_wddx_packet_end(packet);
 					
 	ZVAL_STRINGL(return_value, packet->c, packet->len, 1);
