@@ -1,14 +1,20 @@
 <?php
 
-/* $Id: install-pear.php,v 1.6.2.14.4.1 2005/08/20 18:32:24 cellog Exp $ */
+/* $Id: install-pear.php,v 1.6.2.14.4.7 2006/01/07 17:51:54 cellog Exp $ */
 
+error_reporting(E_ALL);
 $pear_dir = dirname(__FILE__);
 ini_set('include_path', $pear_dir);
+if (function_exists('mb_internal_encoding')) {
+    mb_internal_encoding('ASCII');
+}
+set_time_limit(0);
 include_once 'PEAR.php';
 include_once 'PEAR/Installer.php';
 include_once 'PEAR/Registry.php';
-include_once 'PEAR/Frontend/CLI.php';
-
+include_once 'PEAR/PackageFile.php';
+include_once 'PEAR/Downloader/Package.php';
+include_once 'PEAR/Frontend.php';
 $a = true;
 if (!PEAR::loadExtension('xml')) {
     $a = false;
@@ -25,6 +31,7 @@ if (!$a) {
 $force = false;
 $install_files = array();
 array_shift($argv);
+$debug = false;
 for ($i = 0; $i < sizeof($argv); $i++) {
     $arg = $argv[$i];
     $bn = basename($arg);
@@ -39,6 +46,13 @@ for ($i = 0; $i < sizeof($argv); $i++) {
     } elseif ($arg == '-b') {
         $bin_dir = $argv[$i+1];
         $i++;
+    } elseif ($arg == '-p') {
+        $php_bin = $argv[$i+1];
+        $i++;
+    } elseif ($arg == '--debug') {
+        $debug = 1;
+    } elseif ($arg == '--extremedebug') {
+        $debug = 2;
     }
 }
 
@@ -51,7 +65,11 @@ foreach ($config_layers as $layer) {
     $config->removeLayer($layer);
 }
 $keys = $config->getKeys();
-$config->set('verbose', 0, 'default');
+if ($debug) {
+    $config->set('verbose', 5, 'default');
+} else {
+    $config->set('verbose', 0, 'default');
+}
 // PEAR executables
 if (!empty($bin_dir)) {
     $config->set('bin_dir', $bin_dir, 'default');
@@ -63,43 +81,69 @@ if (!empty($with_dir)) {
     $config->set('doc_dir', $with_dir . $ds . 'doc', 'default');
     $config->set('data_dir', $with_dir . $ds . 'data', 'default');
     $config->set('test_dir', $with_dir . $ds . 'test', 'default');
+    if (!is_writable($config->get('cache_dir'))) {
+        include_once 'System.php';
+        $cdir = System::mktemp(array('-d', 'pear'));
+        if (PEAR::isError($cdir)) {
+            $ui->outputData("[PEAR] cannot make new temporary directory: " . $cdir);
+            die(1);
+        }
+        $oldcachedir = $config->get('cache_dir');
+        $config->set('cache_dir', $cdir);
+    }
 }
-/* Print PEAR Conf (useful for debuging do NOT REMOVE)
-sort($keys);
-foreach ($keys as $key) {
-    echo $config->getPrompt($key) . ": " . $config->get($key) . "\n";
+if (!empty($php_bin)) {
+    $config->set('php_bin', $php_bin);
+}
+/* Print PEAR Conf (useful for debuging do NOT REMOVE) */
+if ($debug) {
+    sort($keys);
+    foreach ($keys as $key) {
+        echo $key . '    ' .
+            $config->getPrompt($key) . ": " . $config->get($key, null, 'default') . "\n";
+    }
+    if ($debug == 2) { // extreme debugging
+        exit;
+    }
 }
-exit;
 // end print
-//*/
 
 $php_dir = $config->get('php_dir');
 $options = array();
+$options['upgrade'] = true;
 $install_root = getenv('INSTALL_ROOT');
 if (!empty($install_root)) {
-    $options['installroot'] = $install_root;
-    $reg_dir = $install_root . $php_dir;
+    $options['packagingroot'] = $install_root;
+    $reg = &new PEAR_Registry($options['packagingroot']);
 } else {
-    $reg_dir = $php_dir;
+    $reg = $config->getRegistry('default');
 }
 
-$reg = &new PEAR_Registry($reg_dir);
-$ui = &new PEAR_Frontend_CLI();
+$ui = &PEAR_Frontend::singleton('PEAR_Frontend_CLI');
 $installer = &new PEAR_Installer($ui);
-//$installer->registry = &$reg; // This should be changed in Installer/Registry
+$pkg = &new PEAR_PackageFile($config, $debug);
 
 foreach ($install_files as $package => $instfile) {
+    $info = &$pkg->fromAnyFile($instfile, PEAR_VALIDATE_INSTALLING);
+    if (PEAR::isError($info)) {
+        $ui->outputData(sprintf("[PEAR] %s: %s", $package,
+            $info->getMessage()));
+        continue;
+    }
+    $new_ver = $info->getVersion();
+    $downloaderpackage = &new PEAR_Downloader_Package($installer);
+    $err = $downloaderpackage->initialize($instfile);
+    if (PEAR::isError($err)) {
+        $ui->outputData(sprintf("[PEAR] %s: %s", $package, $err->getMessage()));
+        continue;
+    }
     if ($reg->packageExists($package)) {
-        $info = $installer->infoFromAny($instfile);
-        if (PEAR::isError($info)) {
-            $ui->outputData(sprintf("[PEAR] %s: %s", $package, $info->getMessage()));
-            continue;
-        }
-        $new_ver = $info['version'];
         $old_ver = $reg->packageInfo($package, 'version');
         if (version_compare($new_ver, $old_ver, 'gt')) {
-            $options['upgrade'] = true;
-            $err = $installer->install($instfile, $options);
+            $installer->setOptions($options);
+            $dp = array(&$downloaderpackage);
+            $installer->setDownloadedPackages($dp);
+            $err = $installer->install($downloaderpackage, $options);
             if (PEAR::isError($err)) {
                 $ui->outputData(sprintf("[PEAR] %s: %s", $package, $err->getMessage()));
                 continue;
@@ -108,7 +152,10 @@ foreach ($install_files as $package => $instfile) {
         } else {
             if ($force) {
                 $options['force'] = true;
-                $err = $installer->install($instfile, $options);
+                $installer->setOptions($options);
+                $dp = array(&$downloaderpackage);
+                $installer->setDownloadedPackages($dp);
+                $err = $installer->install($downloaderpackage, $options);
                 if (PEAR::isError($err)) {
                     $ui->outputData(sprintf("[PEAR] %s: %s", $package, $err->getMessage()));
                     continue;
@@ -120,12 +167,14 @@ foreach ($install_files as $package => $instfile) {
         }
     } else {
         $options['nodeps'] = true;
-        $err = $installer->install($instfile, $options);
+        $installer->setOptions($options);
+        $dp = array(&$downloaderpackage);
+        $installer->setDownloadedPackages($dp);
+        $err = $installer->install($downloaderpackage, $options);
         if (PEAR::isError($err)) {
             $ui->outputData(sprintf("[PEAR] %s: %s", $package, $err->getMessage()));
             continue;
         }
-        $new_ver = $reg->packageInfo($package, 'version');
         $ui->outputData(sprintf("[PEAR] %-15s- installed: %s", $package, $new_ver));
     }
     if ($package == 'PEAR') {
@@ -135,6 +184,9 @@ foreach ($install_files as $package => $instfile) {
                             "'$ufile'. You may probably want to remove it.");
         }
         $config->set('verbose', 1, 'default');
+        if (isset($oldcachedir)) {
+            $config->set('cache_dir', $oldcachedir);
+        }
         foreach ($config->getKeys() as $key) {
             $data[$key] = $config->get($key);
         }
